/*
 * $RCSfile: SharedMemory.cc,v $
 * $Revision: 1.5 $
 *
 * Comments:
 *
 * (C) Copyright ParaSoft Corporation 1996.  All rights reserved.
 * THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF ParaSoft
 * The copyright notice above does not evidence any
 * actual or intended publication of such source code.
 *
 * $Author: mali $          $Locker:  $
 * $Date: 2007-04-23 07:42:33 $
 * $Log: not supported by cvs2svn $
 * Revision 1.4  2007/02/02 15:25:53  mali
 * error handling added
 *
 * Revision 1.3  2007/02/02 12:33:44  mali
 * add usage of ErrorDescription
 *
 * Revision 1.2  2006/12/20 08:19:57  mali
 * shmid can be 0
 *
 * Revision 1.1  2005/08/09 07:52:19  mlyko
 * Original source code.
 *
 *
 */
 
#include <stdio.h>
#include <string.h>
#include <sys/shm.h>
#include <sys/ipc.h>
#include <errno.h>

#include "SharedMemory.h"

/** Shared memory region size */
const static int SM_ALLOC_SIZE = 128 * 1024;

/** Java int identifier */
const static char * JAVA_INT_IDENTIFIER = "I";

/** Key variable name */
const static char * JAVA_KEY_VARIABLE_NAME = "_key";

static ErrorDescription _error;

SharedMemory::SharedMemory() : _shmid(0)
{
    //printf("Constructor !\n");
}

SharedMemory::~SharedMemory()
{
    //printf("Destructor !\n");
}

ErrorDescription& SharedMemory::getErrorDescription()
{
    return _error;
}

int SharedMemory::askJavaForKey(JNIEnv * env, jobject obj)
{
    int key;
    jclass cls = env->GetObjectClass(obj);
    jfieldID fid = env->GetFieldID(cls, JAVA_KEY_VARIABLE_NAME, JAVA_INT_IDENTIFIER);
    if (fid == 0) {
        printf("Failed to get java variable field id.\n");
        key = -1;
    } else {
        key = (jint)env->GetIntField(obj, fid);
    }
    return key;
}

bool SharedMemory::createHandle(JNIEnv * env, jobject obj)
{
    _error.clearError();
    int shmkey = askJavaForKey(env, obj);
    if (shmkey < 0) {
        return false;
    }
    _shmid = shmget(shmkey, SM_ALLOC_SIZE, 0777|IPC_CREAT|IPC_EXCL);
    if (_shmid == -1) {
        _error.setError(errno, "shmget", "Error during creating shared memory handler!");
    }
    printf("createHandle: shmid: %d\n", _shmid);
    return (_shmid >= 0);
}

bool SharedMemory::getHandle(JNIEnv * env, jobject obj)
{
    _error.clearError();
    int shmkey = askJavaForKey(env, obj);
    if (shmkey < 0) {
        return false;
    }
    _shmid = shmget(shmkey, SM_ALLOC_SIZE, 0777);
    if (_shmid == -1) {
        _error.setError(errno, "shmget", "Error during getting shared memory handler!");
    }
    printf("getHandle: shmid: %d\n", _shmid);
    return (_shmid >= 0);
}

bool SharedMemory::setByteArray(JNIEnv * env, jobject obj, jbyteArray byteArray)
{
    _error.clearError();
    void * pSharedMemory = shmat(_shmid, 0, 0);
    if (((void*)-1) == pSharedMemory) {
        _error.setError(errno, "shmat", "Can't create pointer to shared memory!");
        return false;
    }
    jsize len = env->GetArrayLength(byteArray);
    if (len > SM_ALLOC_SIZE) {
        len = SM_ALLOC_SIZE;
    }
    jbyte * byteArrayBody = env->GetByteArrayElements(byteArray, 0);
    memcpy(pSharedMemory, byteArrayBody, len);
    env->ReleaseByteArrayElements(byteArray, byteArrayBody, 0);
    int control = detach(pSharedMemory);
    if (control != 0) {
        printf("Can't detach shared memory (possible memory leak)\n");
    }
    return true;
}

bool SharedMemory::getByteArray(JNIEnv * env, jobject obj, jbyteArray byteArray)
{
    _error.clearError();
    int len = env->GetArrayLength(byteArray);
    if (len > SM_ALLOC_SIZE) {
        printf("Size out of range.");
        return false;
    }
    void * pSharedMemory = shmat(_shmid, 0, 0);
    if (((void*)-1) == pSharedMemory) {
        _error.setError(errno, "shmat", "Can't create pointer to shared memory!");
        return false;
    }
    env->SetByteArrayRegion(byteArray, 0, len, (jbyte *)pSharedMemory);
    int control = detach(pSharedMemory);
    if (control != 0) {
        printf("Can't detach shared memory (possible memory leak)\n");
    }
    return true;
}

bool SharedMemory::unlockSharedMemory()
{
    _error.clearError();
    int result = shmctl(_shmid, IPC_RMID, NULL);
    if (result == -1) {
        _error.setError(errno, "shmctl");
        return false;
    }
    _shmid = -1;
    return true;
}

int SharedMemory::detach(void * pSharedMemory)
{
    // currently do not set error - because according to man error
    // is possble only when pointer point to incorrect value,
    // and in such case we can do nothing...
#ifdef EXP_SOLARIS
    return shmdt((char *)pSharedMemory);
#else
    return shmdt(pSharedMemory);
#endif
}
