<?xml version="1.0"?>
<!-- in case of any change in xsl/xml file modify all connected xsl and xml files from this package  -->
<xsl:stylesheet version="1.1" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:saxon="http://icl.com/saxon" xmlns:textutil="textutil" xmlns:graph="graph" xmlns:results="results" extension-element-prefixes="saxon">

    <!-- ================ print security menu ======================= -->
    <xsl:template name="display_security_menu">

        <xsl:variable name="href" saxon:assignable="yes"/>
        <saxon:assign name="href">#sec</saxon:assign>

        <xsl:if test="$security > 0">
            <xsl:variable name="caption" saxon:assignable="yes" />
            <saxon:assign name="caption" select="$RES_PARAMETER_API_SECURITY_ISSUES" />
            <xsl:call-template name="display_linked_menu_item">
                <xsl:with-param name="text" select="$caption" />
                <xsl:with-param name="link" select="$href" />
            </xsl:call-template>
        </xsl:if>
    </xsl:template>

    <!-- =================  print security summary statistics ============= -->

    <xsl:template name="security_summary_stats">
        <xsl:variable name="label" saxon:assignable="yes"></xsl:variable>
        <saxon:assign name="label" select="$security_issues"/>
        <xsl:value-of select="$label"/>
    </xsl:template>

    <!-- ================= print security header ============================ -->
    <xsl:template name="security_header">

        <table width="{$width}" border="0" bordercolor="{$func_bdrcolor}" cellpadding="0" cellspacing="0">
            <xsl:if test="($report_format_mode='pdf')">
                <xsl:attribute name="cols">15% 70% 15%</xsl:attribute>
            </xsl:if>
            <tr>
                <td width="15%"></td>
                <td align="center">
                    <div class="header"><xsl:value-of select="$RES_PARAMETER_API_SECURITY_ISSUES" /></div>
                </td>
                <td width="15%" align="right" valign="bottom">
                </td>
            </tr>
        </table>
    </xsl:template>
    
    <!-- ================ count risk confidence combination ======================= -->
    <saxon:function name="results:aggregateSecurityCount">
        <xsl:param name="risk_level" />
        <xsl:param name="confidence_level" />
        
        <xsl:variable name="count" saxon:assignable="yes">0</xsl:variable>
        <xsl:variable name="security_viols" select="/ResultsSession/Exec/ExecViols/FuncViol[@secRef and @confidence=$confidence_level]" />
        <xsl:for-each select="($security_viols)">
            <xsl:variable name="categoryId" select="./@secRef" />
            <xsl:if test="/ResultsSession/Exec/FuncAPISecurity/Categories/Category[@id=$categoryId and @risk=$risk_level]" >
                <saxon:assign name="count" select="$count+1"/>
            </xsl:if>
        </xsl:for-each>

        <saxon:return select="$count"/>
    </saxon:function>
    
    <!-- ================ print risk confidence grid ======================= -->
    <xsl:template name="risk_confidence_grid">

        <xsl:variable name="security_logo" saxon:assignable="yes">rep_cwe_logo.jpg</xsl:variable>
        <xsl:variable name="img_alt" saxon:assignable="yes">CWE</xsl:variable>

        <xsl:if test="($report.organize_security_findings_by) = 'CWE'">
            <!-- variables unused, this is to load the image file  -->
            <xsl:variable name="img_uri" select="concat(($output_dir),($cwe_logo_img))"/>
            <xsl:variable name="img_url" select="translate($img_uri, '\','/')"/>
            <xsl:variable name="res_img" select="graph:publish($img_url, $cwe_logo_img)"/>
        </xsl:if>
        
        <xsl:if test="($report.organize_security_findings_by) = 'OWASP2021'">
            <!-- variables unused, this is to load the image file  -->
            <xsl:variable name="img_uri" select="concat(($output_dir),($owasp_logo_img))"/>
            <xsl:variable name="img_url" select="translate($img_uri, '\','/')"/>
            <xsl:variable name="res_img" select="graph:publish($img_url, $owasp_logo_img)"/>
            <saxon:assign name="security_logo">rep_owasp_logo.png</saxon:assign>
            <saxon:assign name="img_alt">OWASP2021</saxon:assign>
        </xsl:if>

        <xsl:variable name="high_total" saxon:assignable="yes">0</xsl:variable>
        <xsl:variable name="medium_total" saxon:assignable="yes">0</xsl:variable>
        <xsl:variable name="low_total" saxon:assignable="yes">0</xsl:variable>
        <xsl:variable name="info_total" saxon:assignable="yes">0</xsl:variable>
        <table cellspacing="0" cellpadding="0" width="450px">
            <xsl:if test="($report_format_mode='pdf')">
              <xsl:attribute name="cols">10% 26% 16% 16% 16% 16%</xsl:attribute>
            </xsl:if>
            <tr>
                <td class="left_separator_gray top_separator_gray bottom_separator_gray" rowspan="2" colspan="2">
                    <img style="padding: 5px">
                        <xsl:attribute name="alt"><xsl:value-of select="$img_alt"/></xsl:attribute>
                        <xsl:attribute name="src"><xsl:value-of select="$security_logo"/></xsl:attribute>
                    </img>
                </td>
                <td class="singleborder left_separator_gray bottom_separator_gray right_separator_gray top_separator_gray" colspan="4" align="center" bgcolor="#f3f3f4">
                    <b><xsl:value-of select="$RES_PARAMETER_CONFIDENCE" /></b>
                </td>
            </tr>
            <tr>
                <td class="singleborder left_separator_gray bottom_separator_gray" align="center" bgcolor="#f3f3f4" width="75px">
                    <xsl:value-of select="$RES_PARAMETER_HIGH" />
                </td>
                <td class="singleborder left_separator_gray bottom_separator_gray" align="center" bgcolor="#f3f3f4" width="75px">
                    <xsl:value-of select="$RES_PARAMETER_MEDIUM" />
                </td>
                <td class="singleborder left_separator_gray bottom_separator_gray" align="center" bgcolor="#f3f3f4" width="75px">
                    <xsl:value-of select="$RES_PARAMETER_LOW" />
                </td>
                <td class="singleborder left_separator_gray bottom_separator_gray right_separator_gray" align="center" bgcolor="#f3f3f4" width="75px">
                    <xsl:value-of select="$RES_PARAMETER_TOTAL" />
                </td>
            </tr>
            <tr>
                <td class="left_separator_gray bottom_separator_gray" rowspan="4" style="text-align:center; writing-mode: tb;" bgcolor="#f3f3f4" width="30px">
                    <b><xsl:value-of select="$RES_PARAMETER_RISK_LEVEL" /></b>
                </td>
                <td class="singleborder left_separator_gray bottom_separator_gray" align="center" bgcolor="#f3f3f4" width="100px">
                    <xsl:value-of select="$RES_PARAMETER_HIGH" />
                </td>
                <td class="singleborder left_separator_gray bottom_separator_gray" align="center">
                    <xsl:attribute name="bgcolor"><xsl:value-of select="$high_risk_certain"/></xsl:attribute>
                    <xsl:variable name="high_high" select="results:aggregateSecurityCount('High', 'High')" />
                    <saxon:assign name="high_total" select="$high_total+$high_high"/>
                    <xsl:value-of select="$high_high" />
                </td>
                <td class="singleborder left_separator_gray bottom_separator_gray" align="center">
                    <xsl:attribute name="bgcolor"><xsl:value-of select="$high_risk_firm"/></xsl:attribute>
                    <xsl:variable name="high_medium" select="results:aggregateSecurityCount('High', 'Medium')" />
                    <saxon:assign name="high_total" select="$high_total+$high_medium"/>
                    <xsl:value-of select="$high_medium" />
                </td>
                <td class="singleborder left_separator_gray bottom_separator_gray" align="center">
                    <xsl:attribute name="bgcolor"><xsl:value-of select="$high_risk_tentative"/></xsl:attribute>
                    <xsl:variable name="high_low" select="results:aggregateSecurityCount('High', 'Low')" />
                    <saxon:assign name="high_total" select="$high_total+$high_low"/>
                    <xsl:value-of select="$high_low" />
                </td>
                <td class="singleborder left_separator_gray bottom_separator_gray right_separator_gray" align="center"><xsl:value-of select="$high_total" /></td>
            </tr>
            <tr>
                <td class="singleborder left_separator_gray bottom_separator_gray" align="center" bgcolor="#f3f3f4">
                    <xsl:value-of select="$RES_PARAMETER_MEDIUM" />
                </td>
                <td class="singleborder left_separator_gray bottom_separator_gray" align="center">
                    <xsl:attribute name="bgcolor"><xsl:value-of select="$med_risk_certain"/></xsl:attribute>
                    <xsl:variable name="medium_high" select="results:aggregateSecurityCount('Medium', 'High')" />
                    <saxon:assign name="medium_total" select="$medium_total+$medium_high"/>
                    <xsl:value-of select="$medium_high" />
                </td>
                <td class="singleborder left_separator_gray bottom_separator_gray" align="center">
                    <xsl:attribute name="bgcolor"><xsl:value-of select="$med_risk_firm"/></xsl:attribute>
                    <xsl:variable name="medium_medium" select="results:aggregateSecurityCount('Medium', 'Medium')" />
                    <saxon:assign name="medium_total" select="$medium_total+$medium_medium"/>
                    <xsl:value-of select="$medium_medium" />
                </td>
                <td class="singleborder left_separator_gray bottom_separator_gray" align="center">
                    <xsl:attribute name="bgcolor"><xsl:value-of select="$med_risk_tentative"/></xsl:attribute>
                    <xsl:variable name="medium_low" select="results:aggregateSecurityCount('Medium', 'Low')" />
                    <saxon:assign name="medium_total" select="$medium_total+$medium_low"/>
                    <xsl:value-of select="$medium_low" />
                </td>
                <td class="singleborder left_separator_gray bottom_separator_gray right_separator_gray" align="center"><xsl:value-of select="$medium_total" /></td>
            </tr>
            <tr>
                <td class="singleborder left_separator_gray bottom_separator_gray" align="center" bgcolor="#f3f3f4">
                    <xsl:value-of select="$RES_PARAMETER_LOW" />
                </td>
                <td class="singleborder left_separator_gray bottom_separator_gray" align="center">
                    <xsl:attribute name="bgcolor"><xsl:value-of select="$low_risk_certain"/></xsl:attribute>
                    <xsl:variable name="low_high" select="results:aggregateSecurityCount('Low', 'High')" />
                    <saxon:assign name="low_total" select="$low_total+$low_high"/>
                    <xsl:value-of select="$low_high" />
                </td>
                <td class="singleborder left_separator_gray bottom_separator_gray" align="center">
                    <xsl:attribute name="bgcolor"><xsl:value-of select="$low_risk_firm"/></xsl:attribute>
                    <xsl:variable name="low_medium" select="results:aggregateSecurityCount('Low', 'Medium')" />
                    <saxon:assign name="low_total" select="$low_total+$low_medium"/>
                    <xsl:value-of select="$low_medium" />
                </td>
                <td class="singleborder left_separator_gray bottom_separator_gray" align="center">
                    <xsl:attribute name="bgcolor"><xsl:value-of select="$low_risk_tentative"/></xsl:attribute>
                    <xsl:variable name="low_low" select="results:aggregateSecurityCount('Low', 'Low')" />
                    <saxon:assign name="low_total" select="$low_total+$low_low"/>
                    <xsl:value-of select="$low_low" />
                </td>
                <td class="singleborder left_separator_gray bottom_separator_gray right_separator_gray" align="center"><xsl:value-of select="$low_total" /></td>
            </tr>
            <tr>
                <td class="singleborder left_separator_gray bottom_separator_gray" align="center" bgcolor="#f3f3f4">
                    <xsl:value-of select="$RES_PARAMETER_INFORMATION" />
                </td>
                <td class="singleborder left_separator_gray bottom_separator_gray" align="center">
                    <xsl:attribute name="bgcolor"><xsl:value-of select="$info_risk_certain"/></xsl:attribute>
                    <xsl:variable name="info_high" select="results:aggregateSecurityCount('Informational', 'High')" />
                    <saxon:assign name="info_total" select="$info_total+$info_high"/>
                    <xsl:value-of select="$info_high" />
                </td>
                <td class="singleborder left_separator_gray bottom_separator_gray" align="center">
                    <xsl:attribute name="bgcolor"><xsl:value-of select="$info_risk_firm"/></xsl:attribute>
                    <xsl:variable name="info_medium" select="results:aggregateSecurityCount('Informational', 'Medium')" />
                    <saxon:assign name="info_total" select="$info_total+$info_medium"/>
                    <xsl:value-of select="$info_medium" />
                </td>
                <td class="singleborder left_separator_gray bottom_separator_gray" align="center">
                    <xsl:attribute name="bgcolor"><xsl:value-of select="$info_risk_tentative"/></xsl:attribute>
                    <xsl:variable name="info_low" select="results:aggregateSecurityCount('Informational', 'Low')" />
                    <saxon:assign name="info_total" select="$info_total+$info_low"/>
                    <xsl:value-of select="$info_low" />
                </td>
                <td class="singleborder left_separator_gray bottom_separator_gray right_separator_gray" align="center"><xsl:value-of select="$info_total" /></td>
            </tr>
        </table>
        <br/>
        <br/>
        <table cellspacing="0" cellpadding="0" width="450px">
            <xsl:if test="($report_format_mode='pdf')">
              <xsl:attribute name="cols">100%</xsl:attribute>
            </xsl:if>
            <tr>
                <td><font class="gray2"><xsl:value-of select="$RES_PARAMETER_CWE_GRID_MSG" /></font></td>
            </tr>
        </table>
    </xsl:template>

    <xsl:key name="securityCat" match="/ResultsSession/Exec/FuncAPISecurity/Categories/Category" use="@cwe" />
    <xsl:key name="owaspId" match="/ResultsSession/Exec/FuncAPISecurity/Categories/Category" use="@owasp" />
    <!-- ================= print security findings ============================ -->
    <xsl:variable name="contentCounter" as="xs:integer" select="1" saxon:assignable="yes"/>
    <xsl:template name="security_findings">
      <table width="{$width}" border="0" bordercolor="{$func_bdrcolor}" cellpadding="0" cellspacing="0" id="security_findings">
        <xsl:if test="($report_format_mode='pdf')">
          <xsl:attribute name="cols">44pt 450pt 44pt</xsl:attribute>
        </xsl:if>
        <tr>
          <td></td>
          <td width="450px" align="center">
            <div class="small_header"><xsl:value-of select="$RES_PARAMETER_SECURITY_FINDINGS"/></div>
          </td>
          <td align="right" valign="bottom"></td>
        </tr>
        <tr>
          <td></td>
          <td width="450px" align="center" colspan="2">
            <table style="border-spacing:10px 5px" width="100%" border="0" bordercolor="{$func_bdrcolor}" cellpadding="0" cellspacing="0">
              <xsl:if test="($report_format_mode='pdf')">
                <xsl:attribute name="cols">20% 10% 70%</xsl:attribute>
              </xsl:if>
              <xsl:if test="($report.organize_security_findings_by) = 'CWE'" >
                  <xsl:apply-templates select="/ResultsSession/Exec/FuncAPISecurity/Categories/Category[not(@cwe = '0') and not(@cwe = '-1') and not(@cwe = '')][generate-id() = generate-id(key('securityCat',@cwe)[1])]" mode="table_content">
                    <xsl:sort select="@cwe" data-type="number"/>
                  </xsl:apply-templates>
                  <xsl:if test="count(/ResultsSession/Exec/FuncAPISecurity/Categories/Category[(@cwe = '0') or (@cwe = '-1') or (@cwe = '')]) > 0">
                    <xsl:call-template name="general_content"/>
                  </xsl:if>
              </xsl:if>
              <xsl:if test="($report.organize_security_findings_by) = 'OWASP2021'" >
                <xsl:apply-templates select="/ResultsSession/Exec/FuncAPISecurity/Categories/Category[not(@owasp = '')][generate-id() = generate-id(key('owaspId',@owasp)[1])]" mode="table_content">
                    <xsl:sort select="@owasp" data-type="text"/>
                  </xsl:apply-templates>
                  <xsl:if test="count(/ResultsSession/Exec/FuncAPISecurity/Categories/Category[@owasp = '']) > 0">
                    <xsl:call-template name="general_content"/>
                  </xsl:if>
              </xsl:if>
            </table>
          </td>
        </tr>
      </table>
      <br/>
      <br/>
    </xsl:template>

    <xsl:template match="/ResultsSession/Exec/FuncAPISecurity/Categories/Category" mode="table_content">
        <xsl:variable name="cwe" select="@cwe" />
        <xsl:variable name="owasp" select="@owasp" />
        <xsl:variable name="owaspId" select="substring($owasp, 12, 3)"  saxon:assignable="yes"/>
        <xsl:if test="starts-with($owaspId, 'A0')">
            <saxon:assign name="owaspId" select="translate($owaspId, '0', '')"/>
        </xsl:if>
        <tr>
            <xsl:if test="($report_format_mode='pdf')">
                <td></td>
            </xsl:if>
            <td style="left:-10px;position:relative" colspan="2" class="security_finding_cat">
                <xsl:if test="($report.organize_security_findings_by) = 'CWE'" >
                    <b><a>
                        <xsl:attribute name="href">#cwe-<xsl:value-of select="@cwe"/></xsl:attribute>
                        <xsl:value-of select="$RES_PARAMETER_CWE"/>-<xsl:value-of select="@cwe"/>
                    </a></b>
                </xsl:if>
                <xsl:if test="($report.organize_security_findings_by) = 'OWASP2021'" >
                    <b><a>
                        <xsl:attribute name="href">#<xsl:value-of select="@owasp"/></xsl:attribute>
                        <xsl:value-of select="$RES_PARAMETER_OWASP"/>&#160;<xsl:value-of select="$owaspId"/>:2021</a></b>
                </xsl:if>
            </td>
        </tr>
        <xsl:if test="($report.organize_security_findings_by) = 'CWE'" >
            <xsl:apply-templates select="/ResultsSession/Exec/FuncAPISecurity/Categories/Category[@cwe=$cwe]" mode="category_content">
                <xsl:sort select="not(@risk='High')"/>
                <xsl:sort select="not(@risk='Medium')"/>
                <xsl:sort select="not(@risk='Low')"/>
                <xsl:sort select="not(@confidence='High')"/>
                <xsl:sort select="not(@confidence='Medium')"/>
                <xsl:sort select="not(@confidence='Low')"/>
            </xsl:apply-templates>
        </xsl:if>
        <xsl:if test="($report.organize_security_findings_by) = 'OWASP2021'" >
            <xsl:apply-templates select="/ResultsSession/Exec/FuncAPISecurity/Categories/Category[@owasp=$owasp]" mode="category_content">
                <xsl:sort select="not(@risk='High')"/>
                <xsl:sort select="not(@risk='Medium')"/>
                <xsl:sort select="not(@risk='Low')"/>
                <xsl:sort select="not(@confidence='High')"/>
                <xsl:sort select="not(@confidence='Medium')"/>
                <xsl:sort select="not(@confidence='Low')"/>
            </xsl:apply-templates>
        </xsl:if>
    </xsl:template>

    <xsl:template name="general_content">
        <tr>
            <xsl:if test="($report_format_mode='pdf')">
                <td></td>
            </xsl:if>
            <td style="left:-10px;position:relative" colspan="2" class="security_finding_cat">
                <b><a href="#general"><xsl:value-of select="$RES_PARAMETER_GENERAL"/></a></b>
            </td>
        </tr>

        <xsl:if test="($report.organize_security_findings_by) = 'CWE'" >
            <xsl:apply-templates select="/ResultsSession/Exec/FuncAPISecurity/Categories/Category[(@cwe = '0') or (@cwe = '-1') or (@cwe = '')]" mode="category_content">
                <xsl:sort select="not(@risk='High')"/>
                <xsl:sort select="not(@risk='Medium')"/>
                <xsl:sort select="not(@risk='Low')"/>
            </xsl:apply-templates>
        </xsl:if>
        <xsl:if test="($report.organize_security_findings_by) = 'OWASP2021'" >
            <xsl:apply-templates select="/ResultsSession/Exec/FuncAPISecurity/Categories/Category[@owasp = '']" mode="category_content">
                <xsl:sort select="not(@risk='High')"/>
                <xsl:sort select="not(@risk='Medium')"/>
                <xsl:sort select="not(@risk='Low')"/>
            </xsl:apply-templates>
        </xsl:if>
    </xsl:template>

    <xsl:template match="/ResultsSession/Exec/FuncAPISecurity/Categories/Category" mode="category_content">
      <xsl:variable name="instances" saxon:assignable="yes" select="0"/>
      <xsl:variable name="currentCategoryId" select="@id" />
      <xsl:for-each select="/ResultsSession/Exec/ExecViols/FuncViol[@secRef=$currentCategoryId]">
        <xsl:sort select="not(@confidence='High')"/>
        <xsl:sort select="not(@confidence='Medium')"/>
        <xsl:sort select="not(@confidence='Low')"/>
        <saxon:assign name="instances" select="$instances + 1"/>
      </xsl:for-each>
      <tr>
        <td width="5%" align="right" style="padding: 0 5px" class="security_finding_cat">
            <a>
                <xsl:attribute name="href">#sec_finding_<xsl:value-of select="$contentCounter"/></xsl:attribute>
                <xsl:value-of select="$contentCounter"/>.
                <xsl:if test="($report_format_mode='pdf')">
                    <xsl:call-template name="make_indentation">
                       <xsl:with-param name="indentation" select="3"/>
                    </xsl:call-template>
                </xsl:if>
            </a>
        </td>
        <td width="10%" align="center" class="risk_confidence_box">
          <xsl:attribute name="bgcolor">
            <xsl:call-template name="get_risk_confidence_color">
              <xsl:with-param name="risk">
                <xsl:value-of select="@risk"/>
              </xsl:with-param>
              <xsl:with-param name="confidence">
                <xsl:value-of select="@confidence"/>
              </xsl:with-param>
            </xsl:call-template>
          </xsl:attribute>
          <span>
            <xsl:choose>
              <xsl:when test="@risk = 'High'">
                <xsl:value-of select="$RES_PARAMETER_HIGH"/>
              </xsl:when>
              <xsl:when test="@risk = 'Medium'">
                <xsl:value-of select="$RES_PARAMETER_MEDIUM"/>
              </xsl:when>
              <xsl:when test="@risk = 'Low'">
                <xsl:value-of select="$RES_PARAMETER_LOW"/>
              </xsl:when>
              <xsl:otherwise>
                <xsl:value-of select="$RES_PARAMETER_INFORMATION"/>
              </xsl:otherwise>
            </xsl:choose>
          </span>
        </td>
        <td width="85%" class="security_finding_cat">
          <xsl:if test="($report_format_mode='pdf')">
              <xsl:call-template name="make_indentation">
                 <xsl:with-param name="indentation" select="3"/>
              </xsl:call-template>
          </xsl:if>
          <a>
            <xsl:attribute name="href">#sec_finding_<xsl:value-of select="$contentCounter"/></xsl:attribute>
            <b><xsl:value-of select="@name"/></b> 
            (<xsl:value-of select="$instances"/><xsl:value-of select="($nbsp)" disable-output-escaping="yes"/>
            <xsl:choose>
              <xsl:when test="$instances = 1">
                <xsl:value-of select="$RES_PARAMETER_INSTANCE"/>
              </xsl:when>
              <xsl:otherwise>
                <xsl:value-of select="$RES_PARAMETER_INSTANCES"/>
              </xsl:otherwise>
            </xsl:choose>)
          </a>
        </td>
      </tr>
      <saxon:assign name="contentCounter" select="$contentCounter + 1"/>
    </xsl:template>

    <xsl:template name="get_risk_confidence_color">
      <xsl:param name="risk"/>
      <xsl:param name="confidence"/>
      <xsl:choose>
         <xsl:when test="$risk = 'High' and $confidence = 'High'">
           <xsl:value-of select="$high_risk_certain"/>
         </xsl:when>
         <xsl:when test="$risk = 'High' and $confidence = 'Medium'">
           <xsl:value-of select="$high_risk_firm"/>
         </xsl:when>
         <xsl:when test="$risk = 'High' and $confidence = 'Low'">
           <xsl:value-of select="$high_risk_tentative"/>
         </xsl:when>
         <xsl:when test="$risk = 'Medium' and $confidence = 'High'">
           <xsl:value-of select="$med_risk_certain"/>
         </xsl:when>
         <xsl:when test="$risk = 'Medium' and $confidence = 'Medium'">
           <xsl:value-of select="$med_risk_firm"/>
         </xsl:when>
         <xsl:when test="$risk = 'Medium' and $confidence = 'Low'">
           <xsl:value-of select="$med_risk_tentative"/>
         </xsl:when>
         <xsl:when test="$risk = 'Low' and $confidence = 'High'">
           <xsl:value-of select="$low_risk_certain"/>
         </xsl:when>
         <xsl:when test="$risk = 'Low' and $confidence = 'Medium'">
           <xsl:value-of select="$low_risk_firm"/>
         </xsl:when>
         <xsl:when test="$risk = 'Low' and $confidence = 'Low'">
           <xsl:value-of select="$low_risk_tentative"/>
         </xsl:when>
         <xsl:when test="$risk = 'Informational' and $confidence = 'High'">
           <xsl:value-of select="$info_risk_certain"/>
         </xsl:when>
         <xsl:when test="$risk = 'Informational' and $confidence = 'Medium'">
           <xsl:value-of select="$info_risk_firm"/>
         </xsl:when>
         <xsl:otherwise>
           <xsl:value-of select="$info_risk_tentative"/>
         </xsl:otherwise>
      </xsl:choose>
    </xsl:template>

    <!-- ================== print security ============================ -->
    <xsl:template name="display_security">
      <xsl:if test="$security > 0">
        <xsl:call-template name="summary_section_header">
          <xsl:with-param name="ref">sec</xsl:with-param>
          <xsl:with-param name="id">security_details_section</xsl:with-param>
          <xsl:with-param name="label">
            <xsl:value-of select="$RES_PARAMETER_API_SECURITY_ISSUES"/>
          </xsl:with-param>
        </xsl:call-template>

        <div id="security_details_section">
          <br />
          <xsl:call-template name="security_header"/>
          <br />
          <table width="100%">
            <xsl:if test="($report_format_mode='pdf')">
              <xsl:attribute name="cols">100%</xsl:attribute>
            </xsl:if>
            <tr>
              <td align="center"><xsl:call-template name="risk_confidence_grid" /></td>
            </tr>
          </table>
          <br/>
          <xsl:call-template name="security_findings"/>
          <xsl:choose>
            <xsl:when test="$security_issues > 0">
              <xsl:call-template name="display_security_table_header"/>
              <xsl:call-template name="display_security_table"/>
            </xsl:when>
            <xsl:otherwise>
              <br/>
              <br/>
              <table width="{$width}" border="0" bordercolor="{$func_bdrcolor}" cellpadding="0" cellspacing="0">
                <xsl:if test="($report_format_mode='pdf')">
                  <xsl:attribute name="cols">15% 70% 15%</xsl:attribute>
                </xsl:if>
                <tr>
                  <td width="15%"></td>
                  <td align="center">
                    <div class="small_header">
                      <xsl:attribute name="style">font-size: large</xsl:attribute>
                      <xsl:value-of select="$RES_PARAMETER_NO_SECURITY_ISSUES_FOUND"/>
                    </div>
                  </td>
                  <td width="15%" align="right" valign="bottom"></td>
                </tr>
              </table>
            </xsl:otherwise>
          </xsl:choose>
        </div>
      </xsl:if>
    </xsl:template>

    <!-- =================  print security table header ============================ -->
    <xsl:template name="display_security_table_header">
      <table width='{$width}' cellpadding='0' cellspacing='0'>
        <xsl:if test="($report_format_mode='pdf')">
          <xsl:attribute name="cols">20% 60% 20%</xsl:attribute>
        </xsl:if>
        <tr>
          <td width="20%"></td>
          <td width="60%" align="center"></td>
          <td width="20%" align="right">
                <xsl:if test="($report_format_mode != 'pdf') and ($report_type != $email_report)">
                  <xsl:call-template name="createStandardSectionControls">
                      <xsl:with-param name="useExpandCollapse" select="true()"/>
                      <xsl:with-param name="expand_func_details" select="'expandCollapseSecurity(true)'"/>
                      <xsl:with-param name="collapse_func_details" select="'expandCollapseSecurity(false)'"/>
                  </xsl:call-template>
              </xsl:if>
          </td>
        </tr>
      </table>
    </xsl:template>

    <!-- ==================  print security table  ============================ -->
    <xsl:variable name="counter" as="xs:integer" select="1" saxon:assignable="yes"/>
    <xsl:template name="display_security_table">
      <xsl:if test="($report_format_mode != 'pdf') and ($report_type != $email_report)">
      <style type="text/css">
        TABLE.security_category_separator{background: #757575}
        TABLE.security_issue_separator{padding-bottom: 0}
        TD.risk_confidence_box{padding: 2px 5px}
        TD.security_row_spacing{padding: 5px}
        TD.solution_cell{background-color: <xsl:value-of select="$cell_background_color"/>; vertical-align: top;}
        TD.cwe_link{text-align:center; padding: 5px 8px; text-decoration: underline;}
        TD.cwe_link A:link{color: <xsl:value-of select="$sectionblue_color"/>}
        TD.cwe_link A:active{color: <xsl:value-of select="$sectionblue_color"/>}
        TD.cwe_link A:visited{color: <xsl:value-of select="$sectionblue_color"/>}
        TD.cwe_link A:hover{color: #0093D2}
        DIV.text_box{padding: 10px;}
        DIV.description{margin-left: 25px;}
        TD.instance_firstlevel{padding-left:50px}
        TD.instance_secondlevel{padding-left:65px}
        TD.instance_thirdlevel{padding-left:75px; white-space: pre-line}
        P.description{margin: 5px; line-height: 20px; display:block;}
        P.solution{margin: 4px; display:block;}
        P.reference_info{margin: 4px; white-space: pre-line; word-break: break-all; display:block;}
        P.instance{padding-right: 10px; margin-top: 5px; display:block; word-break: break-all;}
        DIV.caret_short{float: left; margin-top:8px; margin-left:2px;}
        DIV.caret_tall{float: left; margin-top:12px; margin-left:2px;}
        DIV.caret_instance{float: left; margin-top:4px; margin-left:2px;}
        I.caret{border: solid black; border-width: 0 0.1em 0.1em 0; display: inline-block; padding: 2px; vertical-align: middle; cursor:pointer; margin-right: 4px}
        I.wide{margin-right: 10px;}
        I.collapse{transform: rotate(-45deg); -webkit-transform: rotate(-45deg);}
        I.expand{transform: rotate(45deg); -webkit-transform: rotate(45deg);}
        I.caret.wide{border-width: 0 2px 2px 0}
        I.caret.wide.collapse{margin-top: -3px}
        I.caret.wide.expand{margin-top: -5px}
        I.caret.tall.expand{margin-top: -3px}
        P.truncate{overflow:hidden; text-overflow:ellipsis; white-space:nowrap;}
        DIV.flex{display:flex;}
        TD.security_finding_cat A{color: <xsl:value-of select="$sectionblue_color"/>;font-size: 14px;}
        TD.security_finding_cat A:hover{text-decoration: none;color: #0093D2;}
      </style>
      <script language="JavaScript">
        function toggleCaret(caret) {
            if (caret.classList.contains('collapse')) {
                caret.classList.remove('collapse');
                caret.classList.add('expand');
            } else {
                caret.classList.remove('expand');
                caret.classList.add('collapse');
            }
        }
        function toggleContent(id) {
            var body = document.getElementById(id);
            if (body.style.display == 'none') {
                body.style.display = '';
            } else {
                body.style.display = 'none';
            }
        }
        function toggleText(id) {
            var body = document.getElementById(id);
            if (body.classList.contains('truncate')) {
                body.classList.remove('truncate');
            } else {
                body.classList.add('truncate');
            }
        }
        function expandCollapseSecurity(expand) {
             var table = document.getElementById('security_details_table');
             var carets = table.getElementsByClassName('caret');
             for (i = 0; i &lt; carets.length; i++) {
                if (expand) {
                    carets[i].classList.remove('collapse');
                    carets[i].classList.add('expand');
                } else {
                    carets[i].classList.remove('expand');
                    carets[i].classList.add('collapse');
                }
             }
            var items = table.getElementsByClassName('expandable');
            for (i = 0; i &lt; items.length; i++) {
                if (items[i].tagName == 'TBODY') {
                    if (expand) {                    
                        items[i].style.display = '';
                    } else {
                        items[i].style.display = 'none';
                    }
                } else {
                    if (expand) {
                        items[i].classList.remove('truncate');
                    } else {
                        items[i].classList.add('truncate');
                    }
                }
             }
        }
      </script>
      </xsl:if>
        <table align="center" class="singleborder" width="{$authors_width}" cellpadding='0' cellspacing='0' style="table-layout: fixed; margin-top: 5px;" id="security_details_table">
            <xsl:if test="($report_format_mode='pdf')">
                <xsl:attribute name="cols">70% 8% 8% 14%</xsl:attribute>
            </xsl:if>
            <tr bgcolor="{$func_bcolor}">
                <td class="singleborder right_separator" width="70%" align="center">
                    <b><xsl:value-of select="$RES_PARAMETER_ISSUE" /></b>
                </td>
                <td class="singleborder right_separator" width="10%" align="center">
                    <b><xsl:value-of select="$RES_PARAMETER_RISK_LEVEL" /></b>
                </td>
                <td class="singleborder right_separator" width="10%" align="center">
                    <b><xsl:value-of select="$RES_PARAMETER_CWE" /></b>
                </td>
                <td class="singleborder" width="10%" align="center">
                    <b><xsl:value-of select="$RES_PARAMETER_OWASP" /></b>
                </td>
            </tr>
            <xsl:if test="($report.organize_security_findings_by) = 'CWE'" >
                <xsl:apply-templates select="/ResultsSession/Exec/FuncAPISecurity/Categories/Category[not(@cwe = '0') and not(@cwe = '-1') and not(@cwe = '')][generate-id() = generate-id(key('securityCat',@cwe)[1])]" mode="table">
                    <xsl:sort select="@cwe" data-type="number"/>
                </xsl:apply-templates>
                <xsl:if test="count(/ResultsSession/Exec/FuncAPISecurity/Categories/Category[(@cwe = '0') or (@cwe = '-1') or (@cwe = '')]) > 0">
                    <xsl:call-template name="general_table"/>
                </xsl:if>
            </xsl:if>
            <xsl:if test="($report.organize_security_findings_by) = 'OWASP2021'" >
                <xsl:apply-templates select="/ResultsSession/Exec/FuncAPISecurity/Categories/Category[not(@owasp = '')][generate-id() = generate-id(key('owaspId',@owasp)[1])]" mode="table">
                    <xsl:sort select="@owasp" data-type="text"/>
                </xsl:apply-templates>
                <xsl:if test="count(/ResultsSession/Exec/FuncAPISecurity/Categories/Category[@owasp = '']) > 0">
                    <xsl:call-template name="general_table"/>
                </xsl:if>
            </xsl:if>
        </table>
    </xsl:template>

    <xsl:template match="/ResultsSession/Exec/FuncAPISecurity/Categories/Category" mode="table">
        <xsl:variable name="cwe" select="@cwe" />
        <xsl:variable name="owasp" select="@owasp" />
        <xsl:variable name="owaspId" select="substring($owasp, 12, 3)"  saxon:assignable="yes"/>
        <xsl:if test="starts-with($owaspId, 'A0')">
            <saxon:assign name="owaspId" select="translate($owaspId, '0', '')"/>
        </xsl:if>
        <tr>
            <td style="font-size: 16px; padding: 5px;">
                <xsl:if test="($report.organize_security_findings_by) = 'CWE'" >
                    <xsl:attribute name="id">cwe-<xsl:value-of select="@cwe"/></xsl:attribute>
                    <b><xsl:value-of select="$RES_PARAMETER_CWE"/>-<xsl:value-of select="@cwe"/></b>
                </xsl:if>
                <xsl:if test="($report.organize_security_findings_by) = 'OWASP2021'" >
                    <xsl:attribute name="id"><xsl:value-of select="@owasp"/></xsl:attribute>
                    <b><xsl:value-of select="$RES_PARAMETER_OWASP"/>&#160;<xsl:value-of select="$owaspId"/>:2021</b>
                </xsl:if>
            </td>
            <td style="text-align: right;" colspan="3">
                <xsl:if test="not($report_type = $email_report)">
                    <xsl:call-template name="back_to_security"/>
                </xsl:if>
            </td>
        </tr>
        <xsl:call-template name="print_secuirty_header_separator"/>
        <xsl:if test="($report.organize_security_findings_by) = 'CWE'" >
            <xsl:apply-templates select="/ResultsSession/Exec/FuncAPISecurity/Categories/Category[@cwe=$cwe]" mode="category_table">
                <xsl:sort select="not(@risk='High')"/>
                <xsl:sort select="not(@risk='Medium')"/>
                <xsl:sort select="not(@risk='Low')"/>
                <xsl:sort select="not(@confidence='High')"/>
                <xsl:sort select="not(@confidence='Medium')"/>
                <xsl:sort select="not(@confidence='Low')"/>
            </xsl:apply-templates>
        </xsl:if>
        <xsl:if test="($report.organize_security_findings_by) = 'OWASP2021'" >
            <xsl:apply-templates select="/ResultsSession/Exec/FuncAPISecurity/Categories/Category[@owasp=$owasp]" mode="category_table">
                <xsl:sort select="not(@risk='High')"/>
                <xsl:sort select="not(@risk='Medium')"/>
                <xsl:sort select="not(@risk='Low')"/>
                <xsl:sort select="not(@confidence='High')"/>
                <xsl:sort select="not(@confidence='Medium')"/>
                <xsl:sort select="not(@confidence='Low')"/>
            </xsl:apply-templates>
        </xsl:if>
        <xsl:call-template name="print_secuirty_header_separator"/>
    </xsl:template>

    <xsl:template name="general_table">
      <tr>
        <td id="general" style="font-size: 16px; padding: 5px;"><b><xsl:value-of select="$RES_PARAMETER_GENERAL"/></b></td>
        <td style="text-align: right;" colspan="2">
          <xsl:if test="not($report_type = $email_report)">
            <xsl:call-template name="back_to_security"/>
          </xsl:if>
        </td>
      </tr>
      <xsl:call-template name="print_secuirty_header_separator"/>
        <xsl:if test="($report.organize_security_findings_by) = 'CWE'" >
            <xsl:apply-templates select="/ResultsSession/Exec/FuncAPISecurity/Categories/Category[(@cwe = '0') or (@cwe = '-1') or (@cwe = '')]" mode="category_table">
                <xsl:sort select="not(@risk='High')"/>
                <xsl:sort select="not(@risk='Medium')"/>
                <xsl:sort select="not(@risk='Low')"/>
            </xsl:apply-templates>
        </xsl:if>
        <xsl:if test="($report.organize_security_findings_by) = 'OWASP2021'" >
        <xsl:apply-templates select="/ResultsSession/Exec/FuncAPISecurity/Categories/Category[@owasp = '']" mode="category_table">
                <xsl:sort select="not(@risk='High')"/>
                <xsl:sort select="not(@risk='Medium')"/>
                <xsl:sort select="not(@risk='Low')"/>
            </xsl:apply-templates>
        </xsl:if>
    </xsl:template>

    <xsl:template match="/ResultsSession/Exec/FuncAPISecurity/Categories/Category" mode="category_table">
      <xsl:variable name="currentCategoryId" select="@id" />
      <xsl:for-each select="/ResultsSession/Exec/ExecViols/FuncViol[@secRef=$currentCategoryId]">
          <xsl:sort select="not(@confidence='High')"/>
          <xsl:sort select="not(@confidence='Medium')"/>
          <xsl:sort select="not(@confidence='Low')"/>
      </xsl:for-each>
      <xsl:call-template name="add_secuirty_row_spacing">
        <xsl:with-param name="id" select="concat('sec_finding_',$counter)"/>
      </xsl:call-template>
      <xsl:variable name="instances" select="count(/ResultsSession/Exec/ExecViols/FuncViol[@secRef=$currentCategoryId])"/>
      <xsl:if test="not(position() = 1)">
          <xsl:call-template name="print_category_separator"/>
      </xsl:if>
      <tr>
          <td style="text-align:left; padding: 2px 5px; vertical-align: middle;">
                <xsl:if test="($report_format_mode != 'pdf') and ($report_type != $email_report)">
                  <i class="caret expand wide">
                  <xsl:attribute name="onClick">
                      javascript:toggleCaret(this);toggleContent('<xsl:value-of select="generate-id()"/>')
                  </xsl:attribute>
                  </i>
                </xsl:if>
                  <b><xsl:value-of select="$counter"/>. <xsl:value-of select="@name"/></b>
                  (<xsl:value-of select="$instances"/><xsl:value-of select="($nbsp)" disable-output-escaping="yes"/>
                  <xsl:choose>
                      <xsl:when test="$instances = 1">
                         <xsl:value-of select="$RES_PARAMETER_INSTANCE"/>
                      </xsl:when>
                      <xsl:otherwise>
                         <xsl:value-of select="$RES_PARAMETER_INSTANCES"/>
                      </xsl:otherwise>
                 </xsl:choose>)
          </td>
          <td align="center">
            <xsl:attribute name="bgcolor">
              <xsl:call-template name="get_risk_confidence_color">
                <xsl:with-param name="risk">
                  <xsl:value-of select="@risk"/>
                </xsl:with-param>
                <xsl:with-param name="confidence">
                  <xsl:value-of select="@confidence"/>
                </xsl:with-param>
              </xsl:call-template>
            </xsl:attribute>
            <span>
                <xsl:value-of select="@risk"/>
            </span>
          </td>
          <xsl:choose>
            <xsl:when test="not(@cwe = '0') and not(@cwe = '-1') and not(@cwe = '')">
              <td class="cwe_link">
                <xsl:if test="$report_format_mode = 'pdf'">
                    <xsl:call-template name="make_indentation">
                       <xsl:with-param name="indentation" select="6"/>
                    </xsl:call-template>
                </xsl:if>
                <a target="_blank">
                  <xsl:attribute name="href">https://cwe.mitre.org/data/definitions/<xsl:value-of select="@cwe"/>.html</xsl:attribute>
                  <xsl:value-of select="$RES_PARAMETER_CWE"/>-<xsl:value-of select="@cwe"/><xsl:value-of select="($nbsp)" disable-output-escaping="yes" />&gt;&gt;
                </a>
              </td>
            </xsl:when>
            <xsl:otherwise>
              <td align="center">-</td>
            </xsl:otherwise>
          </xsl:choose>
          <xsl:choose>
            <xsl:when test="not(@owasp = '')">
              <td class="cwe_link">
                  <xsl:if test="$report_format_mode = 'pdf'">
                     <xsl:call-template name="make_indentation">
                        <xsl:with-param name="indentation" select="8"/>
                     </xsl:call-template>
                 </xsl:if>
                 <xsl:choose>
                     <xsl:when test = "contains(@owasp, ',')">
                        <xsl:call-template name="owasp_parse_multiple">
                             <xsl:with-param name="input" select="@owasp"/>
                        </xsl:call-template>
                     </xsl:when>
                     <xsl:otherwise>
                         <xsl:call-template name="owasp_cell">
                             <xsl:with-param name="owaspValue" select="@owasp"/>
                         </xsl:call-template>
                     </xsl:otherwise>
                 </xsl:choose>
              </td>
            </xsl:when>
            <xsl:otherwise>
              <td align="center">-</td>
            </xsl:otherwise>
          </xsl:choose>
      </tr>
    <tbody class="expandable">
        <xsl:attribute name="id"><xsl:value-of select="generate-id()"/></xsl:attribute>
        <xsl:call-template name="add_secuirty_row_spacing"/>
        <xsl:call-template name="description_row"/>
        <xsl:variable name="catRisk" select="@risk"/>
        <xsl:for-each select="/ResultsSession/Exec/ExecViols/FuncViol[@secRef=$currentCategoryId]">
          <xsl:sort select="not(@confidence='High')"/>
          <xsl:sort select="not(@confidence='Medium')"/>
          <xsl:sort select="not(@confidence='Low')"/>
          <tr style="vertical-align: top">
            <td>
              <xsl:call-template name="display_instance_info">
                  <xsl:with-param name="risk" select="$catRisk" />
                  <xsl:with-param name="counter" select="position()" />
                  <xsl:with-param name="url" select="@instanceURL" />
                  <xsl:with-param name="method" select="@method" />
                  <xsl:with-param name="parameter" select="@parameter" />
                  <xsl:with-param name="secAttackId" select="@secAttackRef" />
                  <xsl:with-param name="evidence" select="@evidence" />
                  <xsl:with-param name="confidence" select="@confidence" />
                  <xsl:with-param name="loc" select="@loc" />
                  <xsl:with-param name="otherInfo" select="@otherInfo" />
                  <xsl:with-param name="violTfcRef" select="@tfcRef" />
              </xsl:call-template>
            </td>
          </tr>
        </xsl:for-each>
        <xsl:if test="position() = last()">
         <xsl:call-template name="add_secuirty_row_spacing"/>
     </xsl:if>
    </tbody>
    <saxon:assign name="counter" select="$counter + 1"/>
    </xsl:template>

    <xsl:template name="owasp_parse_multiple">
        <xsl:param name="input"/>
        <xsl:choose>
           <xsl:when test="contains($input, ',')">
              <xsl:variable name="owaspValue" select="substring-before($input, ',')"/>
              <xsl:call-template name="owasp_cell">
                  <xsl:with-param name="owaspValue" select="$owaspValue"/>
                  <xsl:with-param name="commaSeperate">true</xsl:with-param>
              </xsl:call-template>
              <xsl:call-template name="owasp_parse_multiple">
                <xsl:with-param name="input" select="substring-after($input, ',')"/>
              </xsl:call-template>
           </xsl:when>
           <xsl:otherwise>
              <xsl:call-template name="owasp_cell">
                  <xsl:with-param name="owaspValue" select="$input"/>
              </xsl:call-template>
           </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template name="owasp_cell">
        <xsl:param name="owaspValue" />
        <xsl:param name="commaSeperate" />
        <xsl:variable name="owaspLink" saxon:assignable="yes" />
        <xsl:choose>
            <xsl:when test="$owaspValue = 'OWASP_2021_A01'">
                <saxon:assign name="owaspLink">A01_2021-Broken_Access_Control</saxon:assign>
            </xsl:when>
            <xsl:when test="$owaspValue = 'OWASP_2021_A02'">
                <saxon:assign name="owaspLink">A02_2021-Cryptographic_Failures</saxon:assign>
            </xsl:when>
            <xsl:when test="$owaspValue = 'OWASP_2021_A03'">
                <saxon:assign name="owaspLink">A03_2021-Injection</saxon:assign>
            </xsl:when>
            <xsl:when test="$owaspValue = 'OWASP_2021_A04'">
                <saxon:assign name="owaspLink">A04_2021-Insecure_Design</saxon:assign>
            </xsl:when>
            <xsl:when test="$owaspValue = 'OWASP_2021_A05'">
                <saxon:assign name="owaspLink">A05_2021-Security_Misconfiguration</saxon:assign>
            </xsl:when>
            <xsl:when test="$owaspValue = 'OWASP_2021_A06'">
                <saxon:assign name="owaspLink">A06_2021-Vulnerable_and_Outdated_Components</saxon:assign>
            </xsl:when>
            <xsl:when test="$owaspValue = 'OWASP_2021_A07'">
                <saxon:assign name="owaspLink">A07_2021-Identification_and_Authentication_Failures</saxon:assign>
            </xsl:when>
            <xsl:when test="$owaspValue = 'OWASP_2021_A08'">
                <saxon:assign name="owaspLink">A08_2021-Software_and_Data_Integrity_Failures</saxon:assign>
            </xsl:when>
            <xsl:when test="$owaspValue = 'OWASP_2021_A09'">
                <saxon:assign name="owaspLink">A09_2021-Security_Logging_and_Monitoring_Failures</saxon:assign>
            </xsl:when>
            <xsl:when test="$owaspValue = 'OWASP_2021_A10'">
                <saxon:assign name="owaspLink">A10_2021-Server-Side_Request_Forgery_%28SSRF%29</saxon:assign>
            </xsl:when>
       </xsl:choose>
       <xsl:variable name="owaspLabel" select="substring-after(substring-after($owaspValue, '_'), '_')" />
       <a target="_blank">
         <xsl:attribute name="href">https://owasp.org/Top10/<xsl:value-of select="$owaspLink"/></xsl:attribute>
         <xsl:choose>
         <xsl:when test="starts-with($owaspLabel,'A0')">
           <xsl:value-of select="translate($owaspLabel, '0', '')"/>:<xsl:value-of select="substring-before(substring-after($owaspValue, '_'), '_')"/>
         </xsl:when>
         <xsl:otherwise>
           <xsl:value-of select="$owaspLabel"/>:<xsl:value-of select="substring-before(substring-after($owaspValue, '_'), '_')"/>
         </xsl:otherwise>
         </xsl:choose>
         <xsl:value-of select="($nbsp)" disable-output-escaping="yes" />&gt;&gt;
         <xsl:if test="not($commaSeperate = '')"><br/></xsl:if>
       </a>
    </xsl:template>

    <xsl:template name="display_reference_url">
        <xsl:param name="url" />
            <xsl:element name="a">
                <xsl:attribute name="href"><xsl:value-of select="$url"/></xsl:attribute>
                <xsl:attribute name="class">blue_link</xsl:attribute>
                <xsl:value-of select="$url"/>
            </xsl:element>
    </xsl:template>

    <xsl:template name="parse_reference_url">
        <xsl:param name="text" />
        <xsl:param name="separator" />
        <xsl:choose>
            <xsl:when test="not(contains($text, $separator))">
                <xsl:call-template name="display_reference_url" >
                    <xsl:with-param name="url" select="$text" />
                </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
                <xsl:call-template name="display_reference_url" >
                    <xsl:with-param name="url" select="substring-before($text, $separator)" />
                </xsl:call-template>
                <xsl:value-of select="'&#xa;'" disable-output-escaping="yes" />
                <xsl:call-template name="parse_reference_url">
                    <xsl:with-param name="text" select="substring-after($text, $separator)"/>
                    <xsl:with-param name="separator" select="$separator" />
                </xsl:call-template>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template name="description_row">
        <xsl:variable name="id" select="generate-id()"/>
        <tr>
            <td style="vertical-align: top;">
                <div class="text_box description">
                    <div><xsl:value-of select="$RES_PARAMETER_DESCRIPTION"/></div>
                    <div>
                        <xsl:choose>
                            <xsl:when test="($report_format_mode != 'pdf') and ($report_type != $email_report)">
                                <div class="caret_tall"><i class="caret expand" onclick="javascript:toggleCaret(this);toggleText('{$id}_description')"></i></div>
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:call-template name="make_indentation">
                                     <xsl:with-param name="indentation" select="4"/>
                                </xsl:call-template>
                            </xsl:otherwise>
                        </xsl:choose>
                        <div class="flex"><p class="description expandable" id="{$id}_description"><font class="gray2">
                            <xsl:choose>
                                <xsl:when test="$report_format_mode='pdf'">
                                    <xsl:call-template name="make_indentation">
                                         <xsl:with-param name="indentation" select="4"/>
                                    </xsl:call-template>
                                    <xsl:call-template name="split_if_needed">
                                       <xsl:with-param name="value" select="@desc"/>
                                       <xsl:with-param name="length" select="100"/>
                                       <xsl:with-param name="indent" select="1"/>
                                   </xsl:call-template>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:value-of select="@desc" disable-output-escaping="yes"/>
                                </xsl:otherwise>
                            </xsl:choose>
                        </font></p></div>
                    </div>
                </div>
            </td>
            <td class="solution_cell" colspan="3">
                <xsl:if test="($report_format_mode != 'pdf') and ($report_type != $email_report)">
                    <xsl:attribute name="rowspan">2</xsl:attribute>
                </xsl:if>
                <div class="text_box">
                    <div><xsl:value-of select="$RES_PARAMETER_SOLUTION"/></div>
                    <div>
                        <xsl:if test="($report_format_mode != 'pdf') and ($report_type != $email_report)">
                            <div class="caret_short"><i class="caret expand" onclick="javascript:toggleCaret(this);toggleText('{$id}_solution')"></i></div>
                        </xsl:if>
                        <div class="flex"><p class="solution expandable" id="{$id}_solution"><font class="gray2">
                            <xsl:choose>
                                <xsl:when test="$report_format_mode='pdf'">
                                    <xsl:call-template name="make_indentation">
                                         <xsl:with-param name="indentation" select="4"/>
                                    </xsl:call-template>
                                    <xsl:call-template name="split_if_needed">
                                       <xsl:with-param name="value" select="@solution"/>
                                       <xsl:with-param name="length" select="40"/>
                                       <xsl:with-param name="indent" select="1"/>
                                   </xsl:call-template>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:value-of select="@solution" disable-output-escaping="yes"/>
                                </xsl:otherwise>
                            </xsl:choose>
                        </font></p></div>
                    </div>
                </div>
                <xsl:if test="($report_format_mode = 'pdf')">
                    <p><xsl:value-of select="($nbsp)" disable-output-escaping="yes" /></p>
                </xsl:if>
                <div class="text_box">
                    <div><xsl:value-of select="$RES_PARAMETER_REFERENCE"/></div>
                    <div>
                        <xsl:if test="($report_format_mode != 'pdf') and ($report_type != $email_report)">
                            <div class="caret_short"><i class="caret collapse" onclick="javascript:toggleCaret(this);toggleText('{$id}_reference')"></i></div>
                        </xsl:if>
                        <div class="flex"><p class="reference_info truncate expandable" id="{$id}_reference">
                            <xsl:choose>
                                <xsl:when test="$report_format_mode='pdf'">
                                  <font class="gray2">
                                    <xsl:call-template name="make_indentation">
                                         <xsl:with-param name="indentation" select="4"/>
                                    </xsl:call-template>
                                    <xsl:call-template name="split_if_needed">
                                       <xsl:with-param name="value" select="@refInfo"/>
                                       <xsl:with-param name="length" select="40"/>
                                       <xsl:with-param name="indent" select="1"/>
                                   </xsl:call-template>
                                 </font>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:call-template name="parse_reference_url" >
                                        <xsl:with-param name="text" select="@refInfo" />
                                        <xsl:with-param name="separator" select="'&#xa;'" />
                                    </xsl:call-template>
                                </xsl:otherwise>
                             </xsl:choose>
                        </p></div>
                    </div>
                </div>
            </td>
        </tr>
    </xsl:template>
    <xsl:template name="display_instance_info">
        <xsl:param name="risk" />
        <xsl:param name="counter" />
        <xsl:param name="url" />
        <xsl:param name="method"/>
        <xsl:param name="parameter" />
        <xsl:param name="secAttackId"/>
        <xsl:param name="evidence"/>
        <xsl:param name="confidence" />
        <xsl:param name="loc"/>
        <xsl:param name="otherInfo"/>
        <xsl:param name="violTfcRef" />
        <xsl:variable name="id" select="generate-id()"/>
        <table align="center" class="singleborder" width="{$authors_width}" cellpadding='0' cellspacing='0' style="table-layout:fixed">
            <xsl:if test="($report_format_mode='pdf')">
              <xsl:attribute name="cols">50% 12% 8% 35%</xsl:attribute>
            </xsl:if>
            <tr>
                <td width="50%" style="position:relative;left:30px">
                    <b><xsl:value-of select="$counter"/>: <xsl:value-of select="$RES_INSTANCE_URL" /></b>
                </td>
                <td width="50px">
                    <xsl:value-of select="$RES_PARAMETER_CONFIDENCE" />:
                </td>
                <td width="50px" align="center" style="position:relative;left:17px;padding:2px 13px;">
                    <xsl:attribute name="bgcolor">
                            <xsl:call-template name="get_risk_confidence_color">
                              <xsl:with-param name="risk">
                                <xsl:value-of select="$risk"/>
                              </xsl:with-param>
                              <xsl:with-param name="confidence">
                                <xsl:value-of select="$confidence"/>
                              </xsl:with-param>
                            </xsl:call-template>
                        </xsl:attribute>
                        <span>
                            <xsl:choose>
                              <xsl:when test="$confidence = 'High'">
                                <xsl:value-of select="$RES_PARAMETER_HIGH"/>
                              </xsl:when>
                              <xsl:when test="$confidence = 'Medium'">
                                <xsl:value-of select="$RES_PARAMETER_MEDIUM"/>
                              </xsl:when>
                              <xsl:otherwise>
                                <xsl:value-of select="$RES_PARAMETER_LOW"/>
                              </xsl:otherwise>
                            </xsl:choose>
                       </span>
                </td>
                <td></td>
            </tr>
            <xsl:call-template name="add_secuirty_row_spacing"/>
         </table>
         <table align="center" class="singleborder" width="{$authors_width}" cellpadding='0' cellspacing='0' style="table-layout:fixed">
            <xsl:if test="($report_format_mode='pdf')">
              <xsl:attribute name="cols">50% 50%</xsl:attribute>
            </xsl:if>
            <tr>
                <td class="instance_firstlevel" colspan="2">
                    <div>
                        <xsl:if test="($report_format_mode != 'pdf') and ($report_type != $email_report)">
                            <div class="caret_instance"><i class="caret collapse" onclick="javascript:toggleCaret(this);toggleText('{$id}_url');toggleContent('{$id}')"></i><xsl:value-of select="($nbsp)" disable-output-escaping="yes" /></div>
                        </xsl:if>
                        <div class="flex">
                            <p class="instance truncate expandable" id="{$id}_url">
                                <xsl:choose>
                                    <xsl:when test="($report_format_mode='pdf')">
                                        <xsl:call-template name="make_indentation">
                                              <xsl:with-param name="indentation" select="4"/>
                                        </xsl:call-template>
                                        <xsl:call-template name="split_if_needed">
                                            <xsl:with-param name="value" select="$url"/>
                                            <xsl:with-param name="length" select="90"/>
                                            <xsl:with-param name="indent" select="1"/>
                                        </xsl:call-template>
                                    </xsl:when>
                                    <xsl:otherwise>
                                        <font class="gray2"><xsl:value-of select="$url"/></font>
                                    </xsl:otherwise>
                                </xsl:choose>
                                
                            </p>
                        </div>
                    </div>
                </td>
            </tr>
            <tbody id="{$id}" style="display: none;" class="expandable">
                <xsl:if test="$method">
                    <tr>
                        <td class="instance_secondlevel" colspan="2">
                            <xsl:if test="($report_format_mode = 'pdf')">
                                <xsl:call-template name="make_indentation">
                                      <xsl:with-param name="indentation" select="8"/>
                                </xsl:call-template>
                            </xsl:if>
                            <xsl:value-of select="$RES_PARAMETER_METHOD"/>
                        </td>
                    </tr>
                    <tr>
                        <td class="instance_thirdlevel" colspan="2">
                            <xsl:if test="($report_format_mode = 'pdf')">
                                <xsl:call-template name="make_indentation">
                                      <xsl:with-param name="indentation" select="12"/>
                                </xsl:call-template>
                            </xsl:if>
                            <font class="gray2"><xsl:value-of select="$method"/></font>
                        </td>
                    </tr>
                    <xsl:call-template name="add_secuirty_row_spacing"/>
                </xsl:if>
                <xsl:if test="$parameter and not($parameter = '')">
                    <tr>
                        <td class="instance_secondlevel" colspan="2">
                            <xsl:if test="($report_format_mode = 'pdf')">
                                <xsl:call-template name="make_indentation">
                                      <xsl:with-param name="indentation" select="8"/>
                                </xsl:call-template>
                            </xsl:if>
                            <xsl:value-of select="$RES_PARAMETER_PARAMETER"/>
                        </td>
                    </tr>
                    <tr>
                        <td class="instance_thirdlevel" colspan="2">
                            <xsl:if test="($report_format_mode = 'pdf')">
                                <xsl:call-template name="make_indentation">
                                      <xsl:with-param name="indentation" select="12"/>
                                </xsl:call-template>
                            </xsl:if>
                            <font class="gray2"><xsl:value-of select="$parameter"/></font>
                        </td>
                    </tr>
                    <xsl:call-template name="add_secuirty_row_spacing"/>
                </xsl:if>
                <xsl:if test="$secAttackId">
                    <xsl:variable name="securityAttack" select="/ResultsSession/Exec/FuncAPISecurity/Attacks/Attack[@id=$secAttackId]/@data"/>
                    <xsl:if test="not($securityAttack = '')">
                        <tr>
                            <td class="instance_secondlevel" colspan="2">
                                <xsl:if test="($report_format_mode = 'pdf')">
                                <xsl:call-template name="make_indentation">
                                      <xsl:with-param name="indentation" select="8"/>
                                </xsl:call-template>
                            </xsl:if>
                                <xsl:value-of select="$RES_PARAMETER_ATTACK"/>
                            </td>
                        </tr>
                        <tr>
                          <td class="instance_secondlevel" colspan="2">
                             <div>
                                <xsl:choose>
                                    <xsl:when test="($report_format_mode != 'pdf') and ($report_type != $email_report)">
                                        <div class="caret_instance"><i class="caret collapse" onclick="javascript:toggleCaret(this);toggleText('{$id}_attack')"></i><xsl:value-of select="($nbsp)" disable-output-escaping="yes" /></div>
                                        <div class="flex"><p class="instance truncate expandable" id="{$id}_attack"><font class="gray2"><xsl:value-of select="$securityAttack"/></font></p></div>
                                    </xsl:when>
                                    <xsl:otherwise>
                                       <xsl:call-template name="make_indentation">
                                            <xsl:with-param name="indentation" select="12"/>
                                       </xsl:call-template>
                                       <xsl:call-template name="split_if_needed">
                                           <xsl:with-param name="value" select="$securityAttack"/>
                                           <xsl:with-param name="length" select="80"/>
                                           <xsl:with-param name="indent" select="3"/>
                                       </xsl:call-template>
                                    </xsl:otherwise>
                                </xsl:choose>
                            </div>
                          </td>
                      </tr>
                    </xsl:if>
                    <xsl:call-template name="add_secuirty_row_spacing"/>
                </xsl:if>
                <xsl:if test="$evidence and not($evidence = '')">
                    <tr>
                        <td class="instance_secondlevel" colspan="2">
                            <xsl:value-of select="$RES_PARAMETER_EVIDENCE"/>
                        </td>
                    </tr>
                    <tr>
                        <td class="instance_thirdlevel" colspan="2">
                            <font class="gray2"><xsl:value-of select="$evidence"/></font>
                        </td>
                    </tr>
                    <xsl:call-template name="add_secuirty_row_spacing"/>
                </xsl:if>
            </tbody>
            <tr>
                <td class="instance_firstlevel" colspan="2">
                    <xsl:if test="$report_format_mode = 'pdf'">
                        <xsl:call-template name="make_indentation">
                              <xsl:with-param name="indentation" select="4"/>
                        </xsl:call-template>
                    </xsl:if>
                    <xsl:value-of select="$RES_PARAMETER_ASSOCIATED_TEST"/>
                </td>
            </tr>
            <tr>
                <td class="instance_secondlevel" colspan="2">
                    <p class="instance"><font class="gray2">
                        <xsl:if test="$report_format_mode = 'pdf'">
                             <xsl:call-template name="make_indentation">
                                 <xsl:with-param name="indentation" select="8"/>
                            </xsl:call-template>
                        </xsl:if>
                        <xsl:value-of select="$loc"/>
                    </font></p>
                </td>
            </tr>
            <xsl:call-template name="add_secuirty_row_spacing"/>
            <xsl:if test="($otherInfo) and not($otherInfo = '')">
                <tr>
                    <td class="instance_firstlevel" colspan="2">
                        <xsl:if test="$report_format_mode = 'pdf'">
                            <xsl:call-template name="make_indentation">
                                  <xsl:with-param name="indentation" select="4"/>
                            </xsl:call-template>
                        </xsl:if>
                        <xsl:value-of select="$RES_PARAMETER_OTHER_INFORMATION"/>
                    </td>
                </tr>
                <tr>
                    <td class="instance_firstlevel" colspan="2">
                         <div>
                          <xsl:choose>
                            <xsl:when test="$report_format_mode != 'pdf'">
                              <xsl:if test="$report_type != $email_report">
                                <div class="caret_instance"><i class="caret collapse" onclick="javascript:toggleCaret(this);toggleText('{$id}_other')"></i><xsl:value-of select="($nbsp)" disable-output-escaping="yes" /></div>
                              </xsl:if>
                              <div class="flex"><p class="instance truncate expandable" id="{$id}_other"><font class="gray2"><xsl:value-of select="$otherInfo"/></font></p></div>
                            </xsl:when>
                            <xsl:otherwise>
                              <xsl:call-template name="make_indentation">
                                <xsl:with-param name="indentation" select="8"/>
                              </xsl:call-template>
                              <xsl:call-template name="split_if_needed">
                                <xsl:with-param name="value" select="$otherInfo"/>
                                <xsl:with-param name="length" select="90"/>
                                <xsl:with-param name="indent" select="2"/>
                              </xsl:call-template>
                            </xsl:otherwise>
                          </xsl:choose>
                        </div>
                    </td>
                </tr>
            </xsl:if>
            <xsl:variable name="trafficData" select="/ResultsSession/Exec/TrafficData/Traffic[@tfcRef=($violTfcRef)]" />
            <xsl:if test="($trafficData) and not(($report_type) = ($email_report))" >
                <tr>
                    <td style="padding-top: 5px" class="instance_firstlevel">
                        <xsl:call-template name="showTraffic">
                            <xsl:with-param name="trafficRef" select="($violTfcRef)" />
                        </xsl:call-template>
                    </td>
                </tr>
                <xsl:call-template name="add_secuirty_row_spacing"/>
            </xsl:if>
            <xsl:call-template name="add_secuirty_row_spacing"/>
        </table>
    </xsl:template>
    <xsl:template name="back_to_security">
        <xsl:if test="not($report_format_mode='pdf')">
            <a class="blue_link" href="#security_findings"><xsl:value-of select="$RES_BACK_TO_SECURITY"/></a>
            <xsl:value-of select="($nbsp)" disable-output-escaping="yes" />
            <xsl:value-of select="($nbsp)" disable-output-escaping="yes" />
            <xsl:value-of select="($nbsp)" disable-output-escaping="yes" />
        </xsl:if>
    </xsl:template>
    <xsl:template name="print_secuirty_header_separator">
        <tr><td colspan="4">
            <table width="100%" class="header_separator security_category_separator" cellspacing="0" cellpadding="0">
                <xsl:if test="($report_format_mode='pdf')">
                   <xsl:attribute name="cols">100%</xsl:attribute>
                </xsl:if>
                <tr><td align="center"></td></tr>
            </table>
        </td></tr>
    </xsl:template>
    <xsl:template name="print_category_separator">
        <tr>
           <td class="security_row_spacing" colspan="4">
               <table width="100%" class="header_separator" cellspacing="0" cellpadding="0" style="padding-bottom: 0px;">
                    <xsl:if test="($report_format_mode='pdf')">
                       <xsl:attribute name="cols">100%</xsl:attribute>
                    </xsl:if>
                    <tr><td align="center"></td></tr>
                </table>
           </td>
        </tr>
        <xsl:call-template name="add_secuirty_row_spacing"/>
    </xsl:template>
    <xsl:template name="add_secuirty_row_spacing">
      <xsl:param name="id"/>
      <tr>
        <xsl:if test="$id and not($id = '')">
          <xsl:attribute name="id"><xsl:value-of select="$id"/></xsl:attribute>
        </xsl:if>
        <td class="security_row_spacing" colspan="2"></td>
      </tr>
    </xsl:template>

</xsl:stylesheet>
