<?xml version="1.0"?>
<!-- in case of any change in xsl/xml file modify all connected xsl and xml files from this package  -->

<xsl:stylesheet version="1.1" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:saxon="http://icl.com/saxon" xmlns:graph="graph" xmlns:results="results" extension-element-prefixes="saxon">

    <xsl:variable name="isFirstRow" select="true" saxon:assignable="yes"/>
    <xsl:variable name="isEvenRow" saxon:assignable="yes">true</xsl:variable>

    <xsl:include href="res_init.xsl" />
    <xsl:include href="res_build_errors.xsl" />
    <xsl:include href="res_codereview.xsl" />
    <xsl:include href="res_standards.xsl" />
    <xsl:include href="res_generation.xsl" />
    <xsl:include href="res_execution.xsl" />
    <xsl:include href="res_functional.xsl" />
    <xsl:include href="res_api_security.xsl" />
    <xsl:include href="res_api_coverage.xsl" />
    <xsl:include href="res_goals.xsl" />
    <xsl:include href="res_statistics.xsl" />
    
    <!-- ==============  Generate HTML report for manager  ====================  -->               
    <xsl:template match="/">
    <xsl:if test="($report_format_mode='text')">
        <saxon:assign name="nbsp"><xsl:text xml:space="preserve"> </xsl:text></saxon:assign>        
    </xsl:if>
    <xsl:if test="($report_format_mode='pdf')">
        <saxon:assign name="nbsp"><xsl:value-of select="concat('&amp;','#xA0;')"/></saxon:assign>        
        <saxon:assign name="copy"><xsl:value-of select="concat('&amp;','#xA9;')"/></saxon:assign>
    </xsl:if>
     <saxon:assign name="first_back_to_top">true</saxon:assign>

    <xsl:variable name="any_results" 
        select="($coding_standards) > 0 or ($execution) > 0 or ($generation) > 0 or ($codereview) > 0 or ($functional) or ($change_advisor) or ($metrics) > 0"/>        

    <html>
    <head>
    <style type="text/css">
    <xsl:value-of select="($styles)"/>
    </style>
    <xsl:if test="($report_format_mode='pdf')">
        <styles_pdf>
            <xsl:copy-of select="($styles_pdf)"/>
        </styles_pdf>
    </xsl:if>
    <title>
        <xsl:value-of select="($toolDispName)"/><xsl:value-of select="($nbsp)" 
             disable-output-escaping="yes"/><xsl:value-of select="$RES_PARAMETER_REPORT"/><xsl:value-of select="($nbsp)" 
             disable-output-escaping="yes"/>[<xsl:value-of select="($time)"/>]
    </title>

    <xsl:if test="not(($report_type) = ($email_report)) and not($report_format_mode='pdf')">
    <script language="JavaScript">
    <xsl:value-of select="($openWin)"/>
    </script>
    <script language="JavaScript">
    <xsl:value-of select="($sortCodingStandards)"/>
    </script>
    <xsl:if test="($functional) > 0">
        <script language="JavaScript">
            <xsl:value-of select="($showDetailedMessage)"/>
        </script>
        <script language="JavaScript">
            <xsl:value-of select="($showTrafficLog)"/>
        </script>
        <script language="JavaScript">
            <xsl:value-of select="($toggleTestDetails)" />
        </script>
    </xsl:if>

    <script language="JavaScript">
    <xsl:value-of select="($displayTree)"/>
    </script>
    </xsl:if>
    </head>
    <body bgcolor="{$body_color}" style="margin:0;padding:0">
    <table class="singleborder_main" width='100%' cellpadding='0' cellspacing='0'>
    <xsl:if test="($report_format_mode='pdf')">
        <xsl:attribute name="cols">100%</xsl:attribute>
    </xsl:if>
    <tr><td>
    <xsl:call-template name="header_image"/>
    
    <table width='100%' style="padding-bottom:10px;padding-top:0px" cellspacing='0'>
    <xsl:if test="($report_format_mode='pdf')">
        <xsl:attribute name="cols">100%</xsl:attribute>
    </xsl:if>
    <tr>
    <td align="center">
    <!--saxon:assign name="report.developer_errors" select="($report.developer_errors='true') or ($run='gui')"/>
    <xsl:comment>host.name="<xsl:value-of select="($host_name)"/>"</xsl:comment-->
    
    <!-- print header menu -->    
    <!--xsl:call-template name="menu"/-->

    <table width='100%' class="singleborder_blue" style="padding-left:25px;padding-right:25px;padding-bottom:25px;padding-top:5px" cellspacing='0' bgcolor="#FFFFFF" >
    <xsl:if test="($report_format_mode='pdf')">
        <xsl:attribute name="cols">100%</xsl:attribute>
    </xsl:if>
    <tr><td>

    <xsl:call-template name="session_summary"/>

    <xsl:if test="not($report.mail.compact = $links) or not(($report_type) = ($email_report))">
        <xsl:if test="$report.setup.problems='top'">
            <xsl:call-template name="display_build_errors"/>
        </xsl:if>
        <xsl:if test="not($any_results)">
            <xsl:call-template name="empty_report_info"/>
        </xsl:if>
           <xsl:call-template name="display_codereview"/>
           <xsl:call-template name="display_standards"/>
           <xsl:call-template name="display_functional"/>
        <xsl:call-template name="display_security"/>
        <xsl:call-template name="display_generation"/>
           <xsl:call-template name="display_execution"/>
           <xsl:call-template name="display_goals"/>
           <xsl:call-template name="display_statistics"/>
           <xsl:if test="$report.setup.problems='bottom'">
           <xsl:call-template name="display_build_errors"/>
        </xsl:if>
    </xsl:if>
    <xsl:if test="$any_results">
        <br/>
    </xsl:if>
    </td></tr> 
    </table>
    <!-- print footer -->
    <xsl:if test="not($report_type = $email_report)">
    <script language="JavaScript">
    <xsl:value-of select="($init)"/>
    </script>
    </xsl:if>
    </td>
    </tr>
    <!-- xsl:call-template name="footer_image"/>  -->
    </table>
    <xsl:call-template name="footer">
        <xsl:with-param name="include_params">true</xsl:with-param>
    </xsl:call-template>
    </td></tr></table>
    </body>
    </html>

    <!-- generate HTML reports for developers  -->
    <xsl:if test="$any_results">
        <xsl:call-template name="generate_developers_reports"/>
    </xsl:if>
        
    </xsl:template>
    
    <!-- =================  Generate developers reports ============================ -->
    
    <xsl:template name="generate_developers_reports">
        <xsl:for-each select="/ResultsSession/Authors/Author">
        
          <xsl:call-template name="developer_html_report">
             <xsl:with-param name="auth" select="@name"/>
             <xsl:with-param name="authid" select="@id"/>
             <xsl:with-param name="authidx" select="position()"/>
          </xsl:call-template>

       </xsl:for-each>    
    </xsl:template>

    <!-- ========================== Generate HTML report for developer ===================== -->
    
    <xsl:template name="developer_html_report">
    <xsl:param name="auth"/>
    <xsl:param name="authid"/>
    <xsl:param name="authidx"/>
    
     <saxon:assign name="first_back_to_top">true</saxon:assign>

    <xsl:variable name="author_test_classes" 
        select="count(/ResultsSession/Generation/GenerationInformations/GenerationInfo[@auth=($auth)])"/>
    <xsl:variable name="author_test_cases" 
        select="count(/ResultsSession/Generation/GenerationInformations/GenerationInfo[@auth=($auth)]/TestCaseList/TestCase)"/>
        
    <xsl:variable name="author_cs_errors" 
        select="count(/ResultsSession/CodingStandards/StdViols/*[@auth=($auth) and not(@supp='true')])"/>
    <xsl:variable name="author_urgent_cs_errors" 
        select="count(/ResultsSession/CodingStandards/StdViols/*[@auth=($auth) and not(@supp='true') and @urgent='true'])"/>

    <xsl:variable name="author_cs_suppr_msgs" saxon:assignable="yes">0</xsl:variable>
    <xsl:for-each select="/ResultsSession/CodingStandards/Supps/Supp[@suppAuth=($auth)]">
        <xsl:variable name="ref_id" select="@refId"/>
        <xsl:variable name="suppr_msgs"
            select="count(/ResultsSession/CodingStandards/StdViols/*[@suppRef=($ref_id) and @supp='true'])"/>
        <saxon:assign name="author_cs_suppr_msgs" select="$author_cs_suppr_msgs + $suppr_msgs"/>
    </xsl:for-each>

    <xsl:variable name="author_func_errors"
        select="count(/ResultsSession/Exec/ExecViols/*[@auth=($auth) and not(@supp='true')])"/>
        
    <xsl:variable name="author_change_impacts"
        select="count(/ResultsSession/ChangeAdvisor/ChangeImpacts/*[@auth=($auth) and not(@supp='true')])"/>
        
    <xsl:variable name="author_exec_errors" 
        select="count(/ResultsSession/Exec/ExecViols/*[@auth=($auth)])"/>
    <xsl:variable name="author_exec_urgent_errors" 
        select="count(/ResultsSession/Exec/ExecViols/*[@auth=($auth) and @urgent='true'])"/>

    <xsl:variable name="active_user_cr_created_tasks"
            select="count(/ResultsSession/CodeReview/ScannerReviews/Review/CreatedTasks/Task[@owner=($auth) and @new='true' and @active='true'])"/>
    <xsl:variable name="active_user_cr_pending_tasks"
            select="count(/ResultsSession/CodeReview/ScannerReviews/Review/Tasks/Task[@owner=($auth) and (@role='REVIEWER' or @role='OWNER') and @active='true'])"/>
    <xsl:variable name="active_user_cr_monitor_tasks"
            select="count(/ResultsSession/CodeReview/ScannerReviews/Review/Tasks/Task[@owner=($auth) and @role='QAMANAGER' and @active='true'])"/>
    
    <xsl:variable name="author_html" saxon:assignable="yes" select="concat($output_dir,$dev_reports_prefix,$authid,'.html')"/>

    <!--xsl:if test="($report_type) = ($email_report)">
    <saxon:assign name="author_html" select="concat($output_dir,$auth,'_email','.html')"/>
    </xsl:if>
    <xsl:if test="($report_type) = ($tcm_report)">
    <saxon:assign name="author_html" select="concat($output_dir,$auth,'_tcm','.html')"/>
    </xsl:if-->
    
    <xsl:if test="($report.developer_reports = 'true')">
    <xsl:if test="($author_cs_errors > 0) or ($author_exec_errors > 0) 
        or ($active_user_cr_created_tasks > 0) or ($active_user_cr_pending_tasks > 0) or ($active_user_cr_monitor_tasks > 0)
        or (not($author_urgent_cs_errors > 0) and ($report.suppressed_msgs = 'true') and ($author_cs_suppr_msgs > 0)) or ($author_test_cases > 0)
        or ($author_func_errors > 0) or ($author_change_impacts > 0)">
    <xsl:document href="{$author_html}">
    <html>    
    <head>
        <xsl:if test="($report_format_mode='pdf')">
        <styles_pdf>
            <xsl:copy-of select="($styles_pdf)"/>
        </styles_pdf>
        </xsl:if>
    <style type="text/css">
    <xsl:value-of select="($styles)"/>
    </style>

    <title>
    <xsl:value-of select="($auth)"/>  
    <xsl:if test="(($author_cs_errors) > 0) or (not($author_urgent_cs_errors > 0) and ($report.suppressed_msgs = 'true') and ($author_cs_suppr_msgs) > 0)">
        - <xsl:value-of select="$RES_PARAMETER_CODING_STANDARDS_LOWERCASE"/> [ <xsl:value-of select="($author_cs_suppr_msgs)"/> / <xsl:value-of select="($author_cs_errors)"/> / <xsl:value-of select="($author_urgent_cs_errors)"/> ]
    </xsl:if>
    <xsl:if test="($author_func_errors) > 0">
        - <xsl:value-of select="$RES_PARAMETER_FUNCTIONAL_TESTS_LOWERCASE"/> [ <xsl:value-of select="($author_func_errors)"/> ] 
    </xsl:if>
    <xsl:if test="($author_change_impacts) > 0">
        - <xsl:value-of select="$RES_PARAMETER_CHANGE_IMPACTS_LOWERCASE"/> [ <xsl:value-of select="($author_change_impacts)"/> ] 
    </xsl:if>
    <xsl:if test="($author_test_classes) > 0">
        - <xsl:value-of select="$RES_PARAMETER_TEST_GENERATION_LOWERCASE"/> [ <xsl:value-of select="($author_test_cases)"/> ]
    </xsl:if>
    <xsl:if test="($author_exec_errors) > 0">
        - <xsl:value-of select="$RES_PARAMETER_TEST_EXECUTION_LOWERCASE"/> [ <xsl:value-of select="($author_exec_errors)"/> / <xsl:value-of select="($author_exec_urgent_errors)"/> ] 
    </xsl:if>
    </title>
    
    <xsl:if test="not(($report_type) = ($email_report)) and not($report_format_mode='pdf')">
       <xsl:if test="($author_cs_errors) > 0">    
          <script language="JavaScript">
             <xsl:value-of select="($openWin)"/>
          </script>    
          <script language="JavaScript">
             <xsl:value-of select="($sortCodingStandards)"/>
          </script>
       </xsl:if>
       <script language="JavaScript">
           <xsl:value-of select="($showTrafficLog)" />
       </script>
       <script language="JavaScript">
          <xsl:value-of select="($displayTree)"/>
       </script>
       <xsl:if test="($author_func_errors) > 0">
          <script language="JavaScript">
             <xsl:value-of select="($showDetailedMessage)"/>
          </script>
       </xsl:if>
    </xsl:if>   
    </head>

    <body bgcolor="{$body_color}" style="margin:0;padding:0">
    <table class="singleborder_main" width='100%' cellpadding='0' cellspacing='0'>
    <xsl:if test="($report_format_mode='pdf')">
        <xsl:attribute name="cols">100%</xsl:attribute>
    </xsl:if>
    <tr><td>
    <xsl:call-template name="header_image"/>
    <table width='100%' cellspacing='0'>
    <xsl:if test="($report_format_mode='pdf')">
        <xsl:attribute name="cols">100%</xsl:attribute>
    </xsl:if>
    <tr>
    <td align="center">
    <!-- TODO xsl:comment>host.name="<xsl:value-of select="($host_name)"/>"</xsl:comment -->


    <!--xsl:call-template name="menu">
        <xsl:with-param name="auth" select="($auth)"/>
        <xsl:with-param name="authid" select="($authid)"/>
    </xsl:call-template-->
    <table width='100%' class="singleborder_blue" style="padding-left:25px;padding-right:25px;padding-bottom:25px;padding-top:5px" cellspacing='0' bgcolor="#FFFFFF" >
    <xsl:if test="($report_format_mode='pdf')">
        <xsl:attribute name="cols">100%</xsl:attribute>
    </xsl:if>
    <tr><td>

    <xsl:call-template name="session_summary">
        <xsl:with-param name="auth" select="($auth)"/>
        <xsl:with-param name="authid" select="($authid)"/>
        <xsl:with-param name="authidx" select="($authidx)"/>
    </xsl:call-template>
    
    <xsl:if test="not($report.mail.compact = $links) or not(($report_type) = ($email_report))">
    <xsl:if test="not($report_format_mode='pdf')">
    <!-- xsl:variable name="individualMenus">
        <xsl:if test="($active_user_cr_created_tasks > 0) or ($active_user_cr_pending_tasks > 0) or ($active_user_cr_monitor_tasks > 0)">
        <td align="center" bgcolor="{$cs_bcolor}">
            <a class="black_link" href="#cr_{$auth}"><b><xsl:value-of select="$RES_PARAMETER_CODEREVIEW"/></b></a>
        </td>
        </xsl:if>
        <xsl:if test="(($author_cs_errors) > 0) or (not($author_urgent_cs_errors > 0) and ($report.suppressed_msgs = 'true') and ($author_cs_suppr_msgs) > 0)">
        <td align="center" bgcolor="{$cs_bcolor}">
            <a class="black_link" href="#cst_{$auth}"><b><xsl:value-of select="$RES_PARAMETER_CODING_STANDARDS"/></b></a>

        </td>
        </xsl:if>
        <xsl:if test="(($author_func_errors) > 0)">
        <td align="center" bgcolor="{$func_bcolor}">
            <a class="black_link" href="#func_{$auth}"><b><xsl:value-of select="$RES_PARAMETER_FUNCTIONAL_TESTS"/></b></a>
        </td>
        </xsl:if>
        <xsl:if test="(($author_change_impacts) > 0)">
        <td align="center" bgcolor="{$func_bcolor}">
            <a class="black_link" href="#change_{$auth}"><b><xsl:value-of select="$RES_PARAMETER_CHANGE_IMPACT"/></b></a>
        </td>
        </xsl:if>
        <xsl:if test="($author_test_classes) > 0">        
        <td align="center" bgcolor="{$exec_bcolor}">
            <a class="black_link" href="#tgt_{$auth}"><b><xsl:value-of select="$RES_PARAMETER_TEST_GENERATION"/></b></a>

        </td>        
        </xsl:if>
        <xsl:if test="($author_exec_errors) > 0">
        <td align="center" bgcolor="{$exec_bcolor}">
            <a class="black_link" href="#tet_{$auth}"><b><xsl:value-of select="$RES_PARAMETER_TEST_EXECUTION"/></b></a>

        </td>
        </xsl:if>
    </xsl:variable -->
    <!-- xsl:if test="count(saxon:node-set($individualMenus)/td) > 1">
    <table width='{$width}' cellpadding='0' cellspacing='0'>
        <tr>
        <xsl:if test="string-length($test_config_name) = 0">
            <td width="16%" align="center">
                <font class="time"><xsl:value-of select="($time)"/></font>
            </td>
        </xsl:if>
        <xsl:copy-of select="$individualMenus"/>
        </tr>
    </table>
    <br/>
    </xsl:if -->
    </xsl:if>
    </xsl:if>

    <xsl:if test="not($report.mail.compact = $links) or not(($report_type) = ($email_report))">
    <!-- code review section -->

    <xsl:if test="($active_user_cr_created_tasks > 0) or ($active_user_cr_pending_tasks > 0) or ($active_user_cr_monitor_tasks > 0)">
    <xsl:call-template name="codereview_header">
    <xsl:with-param name="ref" select="concat('cr_',$auth)"/>  
    </xsl:call-template>
    <br/>
          
    <xsl:call-template name="codereview_author_details_info">
        <xsl:with-param name="auth" select="($auth)"/>
        <xsl:with-param name="authid" select="($authid)"/>
    </xsl:call-template>
    </xsl:if>       
    
    <!-- coding standards section -->   

    <xsl:if test="(($author_cs_errors) > 0) or (not($author_urgent_cs_errors > 0) and ($report.suppressed_msgs = 'true') and ($author_cs_suppr_msgs) > 0)">
    
    <xsl:variable name="auth_cs_section_id" select="concat('cs_details_section_',$auth)"/>
    <xsl:call-template name="collapsible_section_header">
        <xsl:with-param name="ref" select="concat('cst_',$auth)"/>
        <xsl:with-param name="id" select="($auth_cs_section_id)"/>
        <xsl:with-param name="label" select="$RES_PARAMETER_DETAILS_STATIC_ANALYSIS"/>      
    </xsl:call-template>

    <div id="{$auth_cs_section_id}">
    <xsl:call-template name="cs_header"/>
    <br/>

    <xsl:call-template name="standards_author_details_info">
        <xsl:with-param name="auth" select="($auth)"/>
        <xsl:with-param name="authid" select="($authid)"/>
        <xsl:with-param name="authidx" select="($authidx)"/>
        <xsl:with-param name="author_cs_errors" select="($author_cs_errors)"/>
        <xsl:with-param name="author_urgent_cs_errors" select="($author_urgent_cs_errors)"/>
        <xsl:with-param name="author_cs_suppr_msgs" select="($author_cs_suppr_msgs)"/>
    </xsl:call-template>

    <xsl:if test="($report.suppressed_msgs = 'true') and ($author_cs_suppr_msgs) > 0">
    <xsl:call-template name="supp_messages">
    <xsl:with-param name="auth" select="$auth"/>
    </xsl:call-template>

    </xsl:if>
    </div>
    </xsl:if>

    <!-- functional tests section -->
    <xsl:if test="($functional) > 0 and ($func_errors_nmb) > 0">

        <xsl:variable name="auth_func_section_id" select="concat('func_details_section_',$auth)"/>
        <xsl:call-template name="collapsible_section_header">
           <xsl:with-param name="ref" select="concat('func_',$auth)"/>  
           <xsl:with-param name="id" select="($auth_func_section_id)"/>
           <xsl:with-param name="label" select="$RES_PARAMETER_DETAILS_FUNCTIONAL_TESTS"/>      
        </xsl:call-template>

        <div id="{$auth_func_section_id}">

        <xsl:call-template name="func_header">
            <xsl:with-param name="isFunctional">true</xsl:with-param>
        </xsl:call-template>

        <br/>
        <table width='{$width}'
            cellpadding='0' cellspacing='0'>
        <xsl:if test="($report_format_mode='pdf')">
            <xsl:attribute name="cols">20% 70% 10%</xsl:attribute>
        </xsl:if>
            <xsl:call-template name="func_summary_details">
                <xsl:with-param name="auth" select="($auth)" />
                <xsl:with-param name="authid" select="($authid)" />
                <xsl:with-param name="isFunctional">true</xsl:with-param>
            </xsl:call-template>
        </table>
        </div>
    </xsl:if>

    <!-- change impacts section -->
    <xsl:if test="(($change_impacts_nmb) > 0)">

        <xsl:variable name="auth_change_section_id" select="concat('change_details_section_',$auth)"/>
        <xsl:call-template name="collapsible_section_header">
           <xsl:with-param name="ref" select="concat('change_',$auth)"/>  
           <xsl:with-param name="id" select="($auth_change_section_id)"/>
           <xsl:with-param name="label" select="$RES_PARAMETER_DETAILS_CHANGE_IMPACTS"/>      
        </xsl:call-template>

        <div id="{$auth_change_section_id}">

        <xsl:call-template name="func_header">
            <xsl:with-param name="isFunctional">false</xsl:with-param>
        </xsl:call-template>

        <br/>
        <table width='{$width}' cellpadding='0' cellspacing='0'>
        <xsl:if test="($report_format_mode='pdf')">
            <xsl:attribute name="cols">20% 70% 10%</xsl:attribute>
        </xsl:if>
            <xsl:call-template name="func_summary_details">
                <xsl:with-param name="auth" select="($auth)" />
                <xsl:with-param name="authid" select="($authid)" />
                <xsl:with-param name="isFunctional">false</xsl:with-param>
            </xsl:call-template>
        </table>
        </div>
    </xsl:if>

    <!-- test generation section -->
    <xsl:if test="($author_test_classes) > 0">
    <xsl:if test="($author_exec_errors) > 0 or ($author_cs_errors) > 0">
    <br/>
    </xsl:if>
    <xsl:call-template name="tg_header">
    <xsl:with-param name="ref" select="concat('tgt_',$auth)"/>
    </xsl:call-template>
    <br/>
    <table width='{$width}' cellpadding='0' cellspacing='0'>
        <xsl:if test="($report_format_mode='pdf')">
            <xsl:attribute name="cols">80% 10% 10%</xsl:attribute>
        </xsl:if>
    <xsl:call-template name="tg_author_details_info">
    <xsl:with-param name="auth" select="($auth)"/>
    <xsl:with-param name="mode">single</xsl:with-param>
    </xsl:call-template>
    </table>
    </xsl:if>

    <!-- test execution section -->

    <xsl:if test="($author_exec_errors) > 0 and ($execution) > 0">
       <xsl:if test="($author_cs_errors) > 0">
          <br/>
       </xsl:if>

       <xsl:variable name="auth_te_section_id" select="concat('te_details_section_',$auth)"/>
       <xsl:call-template name="collapsible_section_header">
          <xsl:with-param name="ref" select="concat('tet_',$auth)"/>
          <xsl:with-param name="id" select="($auth_te_section_id)"/>
          <xsl:with-param name="label" select="$RES_PARAMETER_DETAILS_TEST_EXECUTION"/>      
       </xsl:call-template>

       <div id="{$auth_te_section_id}">

       <xsl:call-template name="exec_header"/>
       <br/>
       <xsl:call-template name="execution_author_details_info">
           <xsl:with-param name="auth" select="($auth)"/>
           <xsl:with-param name="authid" select="($authid)"/>
           <xsl:with-param name="authidx" select="($authidx)"/>
           <xsl:with-param name="author_exec_errors" select="($author_exec_errors)"/>
           <xsl:with-param name="author_exec_urgent_errors" select="($author_exec_urgent_errors)"/>
       </xsl:call-template>
       </div>

    </xsl:if>
    </xsl:if>
    <!-- print footer -->
    <br/>
        
    <xsl:if test="not($report_type = $email_report)">
    <script language="JavaScript">
    <xsl:value-of select="($init)"/>
    </script>
    </xsl:if>
    </td>
    </tr>
    <!-- xsl:call-template name="footer_image"/> -->
    </table>
    </td></tr>
    </table>
    <xsl:call-template name="footer">
        <xsl:with-param name="include_params">false</xsl:with-param>
    </xsl:call-template>
    </td>
    </tr>
    </table>
    </body>
    </html>
    </xsl:document>
    </xsl:if>
    </xsl:if>
    </xsl:template>
        
    <!-- =================  print menu ============================ -->
    
    <xsl:template name="menu">
    <xsl:param name="auth"/>
    <xsl:param name="authid"/>
    
    <table width="{$width}" cellpadding="0" cellspacing="0">
    <xsl:if test="($report_format_mode='pdf')">
    <xsl:attribute name="cols">20% 60% 20%</xsl:attribute>
    </xsl:if>
    <tr>
    <td rowspan="2" style="padding-left:20px" align="left" valign="bottom" width="20%">
    <xsl:if test="not($report.mail.compact = $links) or not(($report_type) = ($email_report))">
    <xsl:if test="$report_format_mode='html'">
    <xsl:variable name="individualMenus">
    <xsl:if test="$report.setup.problems='top'">
    <tr><xsl:call-template name="display_build_errors_menu"/></tr>
    </xsl:if>
    <tr>
    <xsl:call-template name="display_codereview_menu">
    <xsl:with-param name="auth" select="$auth"/>
    </xsl:call-template>
    </tr>
    <tr>
    <xsl:call-template name="display_standards_menu">
    <xsl:with-param name="auth" select="$auth"/>
    </xsl:call-template>
    </tr>
    <tr>
        <xsl:call-template name="display_apicoverage_menu"/>
    </tr>    
    <tr>
    <xsl:call-template name="display_functional_menu">
    <xsl:with-param name="auth" select="$auth"/>
    </xsl:call-template>
    </tr>
    <tr>
    <xsl:call-template name="display_security_menu"/>
    </tr>
    <tr>
    <xsl:call-template name="display_generation_menu">
    <xsl:with-param name="auth" select="$auth"/>
    </xsl:call-template>
    </tr>
    <tr>
    <xsl:call-template name="display_execution_menu">
    <xsl:with-param name="auth" select="$auth"/>
    </xsl:call-template>
    </tr>
    <xsl:if test="not($authid)">
    <tr><xsl:call-template name="display_goals_menu"/></tr>
    <tr><xsl:call-template name="display_statistics_menu"/></tr>
    </xsl:if>
    
    <xsl:if test="$report.setup.problems='bottom'">
    <tr><xsl:call-template name="display_build_errors_menu"/></tr>
    </xsl:if>
    </xsl:variable>
    <!-- individual menus removed 
    xsl:if test="count(saxon:node-set($individualMenus)/tr/td) > 1">
    <table width="{$width}" cellpadding="0" cellspacing="0">
    <xsl:if test="($report_format_mode='pdf')">
    <xsl:attribute name="cols">30% 70%</xsl:attribute>
    </xsl:if>
    <xsl:copy-of select="$individualMenus/tr[count(./td) > 0]"/>
    </table>
    </xsl:if -->
    </xsl:if>
    </xsl:if>
    </td>
    </tr>

    </table>
    </xsl:template>

    <!-- =================  report summary info ============================ -->
    
    <xsl:template name="session_summary">
        <xsl:param name="auth"/>
        <xsl:param name="authid"/>
        <xsl:param name="authidx"/>
    
        <table width="100%" cellpadding="0" cellspacing="0">
        <xsl:if test="($report_format_mode='pdf')">
            <xsl:attribute name="cols">50% 50%</xsl:attribute>
        </xsl:if>
        <tr>
        <td colspan="2">          
            <table width="100%" cellpadding="0" cellspacing="0" class="section_header">
            <xsl:if test="($report_format_mode='pdf')">
                <xsl:attribute name="cols">100%</xsl:attribute>
            </xsl:if>            
            <tr><td>
                <table width="100%" cellpadding='0' cellspacing='0'>
                <xsl:if test="($report_format_mode='pdf')">
                    <xsl:attribute name="cols">100%</xsl:attribute>
                </xsl:if>
                   <tr><td class="section_header">
                       <xsl:value-of select="$RES_PARAMETER_SESSION_SUMMARY" />
                     </td></tr>
                </table>
            </td></tr>
            </table>
        </td>
        </tr>

        <tr>
        <td class="summary_cell">
           <table width="100%" cellpadding="0" cellspacing="0" >
               <xsl:if test="($report_format_mode='pdf')">
                <xsl:attribute name="cols">30% 70%</xsl:attribute>
            </xsl:if>
           <tr>
              <td width="20%" class="summary_cell" style="white-space: nowrap; vertical-align: baseline;"><b><xsl:value-of select="$RES_PARAMETER_BUILD_ID"/></b></td>
              <td class="summary_cell" style="word-break: break-word;">
                 <xsl:value-of select="$build_id"/>
              </td>
           </tr>
           <tr>
              <td class="summary_cell" style="white-space: nowrap; vertical-align: baseline;"><b><xsl:value-of select="$RES_PARAMETER_TEST_ID"/></b></td>
              <td class="summary_cell" style="word-break: break-word;"><xsl:if test="string-length($test_config) > 0">
                 <xsl:value-of select="$test_config"/>
              </xsl:if></td>
           </tr>
           <tr>
              <td class="summary_cell" style="white-space: nowrap; vertical-align: baseline;"><b><xsl:value-of select="$RES_PARAMETER_STARTED"/></b></td>
              <td class="summary_cell"><xsl:value-of select="$time"/></td>
           </tr>
           <tr>
              <td class="summary_cell" style="white-space: nowrap; vertical-align: baseline;"><b><xsl:value-of select="$RES_PARAMETER_PERFORMED_ON"/></b></td>
              <td class="summary_cell">
                  <xsl:if test="string-length($user) > 0">
                     <xsl:value-of select="concat($machine,' ',$RES_PARAMETER_BY,' ', $user)"/>
                  </xsl:if>
                  <xsl:if test="string-length($user) = 0">
                     <xsl:value-of select="$machine"/>
                  </xsl:if>
              </td>
           </tr>
           <tr>
              <td class="summary_cell" style="white-space: nowrap; vertical-align: baseline;"><b><xsl:value-of select="$RES_PARAMETER_SESSION_TAG"/></b></td>
              <td class="summary_cell" style="word-break: break-word;">
                  <xsl:value-of select="$sessionTag"/>
              </td>
           </tr>
           <tr>
              <td class="summary_cell" style="white-space: nowrap; vertical-align: baseline;"><b><xsl:value-of select="$RES_PARAMETER_PROJECT"/></b></td>
              <td class="summary_cell">
                  <xsl:value-of select="$dtpProject"/>
              </td>
           </tr>
           </table>
        </td>

        <td class="summary_cell" align="right" valign="top" style="padding-right:60px;">
          <br/>
          <xsl:if test="($codereview > 0) or ($coding_standards > 0) or ($functional > 0) or ($change_advisor > 0) or ($execution > 0) or ( ($statistics) > 0 and ($generate_graphs = 'true') and not($auth))">
          <table width="400px" class="summary_stats_cell" cellpadding="0" bgcolor="#FFFFFF" cellspacing="0">
          <xsl:if test="($report_format_mode='pdf')">
            <xsl:attribute name="cols">100%</xsl:attribute>
            </xsl:if>
          <tr><td>
             <table width="100%" cellpadding="0" bgcolor="#FFFFFF" cellspacing="0">
             <xsl:if test="($report_format_mode='pdf')">
                   <xsl:attribute name="cols">50% 50%</xsl:attribute>
             </xsl:if>
              
               <xsl:if test="($codereview) > 0">   
               <tr>
                  <td class="summary_stats_cell" width="50%">
                     <xsl:call-template name="display_codereview_menu">
                        <xsl:with-param name="auth" select="$auth"/>
                     </xsl:call-template>
                  </td>
                  <td class="summary_stats_cell" width="50%">
                       <xsl:value-of select="($nbsp)" disable-output-escaping="yes"/>  
                  </td>
               </tr>
               </xsl:if>

               <xsl:if test="($coding_standards) > 0">   
               <tr>
                  <td class="summary_stats_cell">
                     <xsl:call-template name="display_standards_menu">
                        <xsl:with-param name="auth" select="$auth"/>
                     </xsl:call-template>
                  </td>
                  <td class="summary_stats_cell">
                     <xsl:call-template name="cs_summary_stats">
                        <xsl:with-param name="auth" select="$auth"/>
                     </xsl:call-template>
                  </td>
               </tr>
               </xsl:if>
               
               <xsl:if test="($functional > 0) or ($change_advisor > 0)">
               <tr>
                  <td class="summary_stats_cell">
                          <xsl:call-template name="display_functional_menu">
                             <xsl:with-param name="auth" select="$auth"/>
                          </xsl:call-template>
                  </td>
                  <td class="summary_stats_cell">
                          <xsl:call-template name="func_tests_summary_stats">
                             <xsl:with-param name="isFunctional">true</xsl:with-param>
                             <xsl:with-param name="auth" select="$auth"/>
                          </xsl:call-template>
                  </td>
               </tr>
              </xsl:if>   

              <xsl:if test="($security) > 0">
               <tr>
                  <td class="summary_stats_cell">
                    <xsl:call-template name="display_security_menu" />
                  </td>
                  <td class="summary_stats_cell">
                    <xsl:call-template name="security_summary_stats" />
                  </td>
               </tr>
              </xsl:if>

              <xsl:if test="($execution) > 0">
               <tr>
                  <td class="summary_stats_cell">
                          <xsl:call-template name="display_execution_menu">
                             <xsl:with-param name="auth" select="$auth"/>
                          </xsl:call-template>
                  </td>
                  <td class="summary_stats_cell">
                          <xsl:call-template name="exec_summary_stats">
                             <xsl:with-param name="auth" select="$auth"/>
                          </xsl:call-template>
                  </td>
               </tr>
              </xsl:if>

              <!--xsl:if test="($statistics) > 0 and ($generate_graphs = 'true') and not($auth)">
               <tr>
                  <td class="summary_stats_cell">                                  
                          <xsl:call-template name="display_statistics_menu"/><xsl:value-of select="($nbsp)" disable-output-escaping="yes"/>
                  </td>
                  <td class="summary_stats_cell">
                      <b><xsl:value-of select="($nbsp)" disable-output-escaping="yes"/></b>  
                  </td>
               </tr>
              </xsl:if-->
                  
             </table>
          </td></tr>
          </table>
          </xsl:if>
        </td>
      </tr>
    </table>

    </xsl:template>
    
    <!-- =================  print empty report info ============================ -->
    
    <xsl:template name="empty_report_info">
       <br/>
       <b><xsl:value-of select="$RES_PARAMETER_NO_RESULTS"/></b>
    </xsl:template>

    <!-- =================  print footer ============================ -->

    <xsl:template name="footer">
    <xsl:param name="include_params"/>

    <xsl:if test="$include_params='true' and (string-length($test_params) > 0)">

    <table style="padding-right:25px;padding-left:25px"  width='100%' cellspacing='0'>
    <xsl:if test="($report_format_mode='pdf')">
        <xsl:attribute name="cols">100%</xsl:attribute>
    </xsl:if>
    <tr>
    <td>
            <table width="{$width}" class="singleborder_blue" bgcolor="#FFFFFF" cellspacing="0" align="center" rules="none">
                <xsl:if test="($report_format_mode='pdf')">
                    <xsl:attribute name="cols">100%</xsl:attribute>
                </xsl:if>
                <tr>
                    <td bgcolor="#FFFFFF" align="center">
                        <div class="small_header"><xsl:value-of select="$RES_PARAMETER_TEST_PARAMETERS"/></div> 
                    </td>
                </tr>
                <tr><td>
                    <xsl:call-template name="print_header_separator"/>
                </td></tr>
                <tr>
                    <td style="" bgcolor="#FFFFFF" align="left">
                        <table class="doubleborder" width="100%" cellpadding="2" cellspacing="0" align="center">
                        <xsl:if test="($report_format_mode='pdf')">
                            <xsl:attribute name="cols">100%</xsl:attribute>
                        </xsl:if>
                        <tr>
                            <td class="tree" align="left">
                                <div id="paramsHeader"><xsl:value-of select="$test_params"/></div>
                            </td>
                        </tr>
                        </table>
                    </td>
                </tr>
            </table>
    </td>
    </tr>
    </table>
            <br />
        </xsl:if>

    <!--table width='100%' bgcolor="{$footer_color}" cellpadding='10' cellspacing='0'>
        <xsl:if test="($report_format_mode='pdf')">
            <xsl:attribute name="cols">100%</xsl:attribute>
        </xsl:if>
    <tr>
    <td align="center"> 
    <font class="white">
    <xsl:value-of select="($copy)" disable-output-escaping="yes"/>
        <xsl:value-of select="$RES_PARAMETER_PARASOFT"/>
    <xsl:value-of select="($nbsp)" disable-output-escaping="yes"/>
    <xsl:value-of select="($toolDispName)"/>
    <xsl:value-of select="($nbsp)" disable-output-escaping="yes"/>
    <xsl:value-of select="($toolVer)"/>
    <xsl:value-of select="($nbsp)" disable-output-escaping="yes"/>
        <xsl:value-of select="$RES_PARAMETER_REPORTIMNG_SYSTEM"/>
    <xsl:value-of select="($nbsp)" disable-output-escaping="yes"/>
    <xsl:if test="string-length($os_name) > 0">   
        <xsl:value-of select="$RES_PARAMETER_OS"/> <xsl:value-of select="($os_name)"/>
    </xsl:if>
    </font> 
    </td>
    </tr>
    </table-->
    </xsl:template>

</xsl:stylesheet>
